ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  program main
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

C  Dan Vimont, July 10, 2022
C
C  Compile the code using:
C  gfortran ENSO_model_COUP.f
C
C  Then run the code using:
C  ./a.out < time_a
C
C  This is the ICM code used in Chiang et al. (2022). The model is the 
C     Intermediate Coupled Model (ICM) of Battisti (1988) (B88). A big thanks
C     to David Battisti for sharing the code. If you do use this code, please
C     send an email to Dan Vimont (dvimont@wisc.edu), and acknowledge both
C     Battisti (1988) and Chiang et al. (2022). 
C
C  Input files include:
C     time_a --> Namelist input file, provides time of run etc.
C     ocean2b.annave.ascii --> Annually averaged mean ocean currents
C     sst.mean.SOM_fixedang_000_EC00.12mo --> Seasonal SST from the EC00 simulation
C     uv992sig.mean.SOM_fixedang_000_EC00.12mo --> Seasonal low-level winds from EC00 simulation
C     uv992sig.anom.SOM_fixedang_000_EC04mEC00.12mo --> Anomalous annual cycle
C       of winds used to force the model (EC04 minus EC00 simulation)
C     FSNS.globanm.SOM_fixedang_000_EC04mEC00.12mo --> mean SST and solar forcing
C       over global domain. Note that global domain is not used to calculate 
C       heating anomalies in this version.
C
C  Note that the code below has been altered enough times that it no longer
C     reproduces the original B88 results. Further, parameters are not the same as
C     the original B88 model.
C 
C     LINEAR OCEAN MODEL - TWO LAYERS - WIND DRIVEN ONLY
c     this version of the model is used to calculate the anomalous
c     cycle of the ocean due to the pseudo winds (the winds needed
c     to give the correct anomalous stress by just squaring the wind
c     we will start the model by reading in the anual cycle of the
c     ocean currents (ubar,vbar,wbar) and sst (tbar)and the anomalous
c     pseudo winds for the composite 3 year R and C ENSO
c     output will be the corse grid scale variables u,v,h,t where
c     u and v are the anomalous mixed layer currents, h the upper layer
c     height anomalies and t the surface temperature anomalies. All
c     variables are dimensional in output.
c
cc now the n-s Kelvin waves  ARE  filtered out of the solution (cddd)
c  now has changes of errors noted by tony; does no plotting
c moved from the Prime on Feb 7 1987
cc now artifically changed to give tsub by h*1.5 (see cho comment)
cc
cc now alpt is 125 days

cc  note the anomalous w advection term is calculated different and
cc  the mean w advection at i=26 is increased by 1.5 in magnitude
cc right now no fields are written like u,v,h,t,etc
cc  look for cddd to put back in
cc This version is ocean2 (the more correct way to calculate w and uses
cc the mean ocean fields 'OCEAN.AVE.FIELD2b'
c  NOTE gama1=gama2=.75
c
      program main
      parameter(m=78,mp=79,n=119,np=120)
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)
      common/filt/ tfil(26,30)
      common/o1/ua(m,n),va(mp,np),wndnon,wndno2,fert2
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/c2/fk(m)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/m2/f(m,n),fert
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common/p1/phi(np)
      common /cl2/mthend(12),day(12)
      common/s10/us(26,30),vs(26,30),hs(26,30),rs,yy(30),cof(n)
      common/coup/uanom(26,30),vanom(26,30)
      integer p,iseed
      real npoamp
      character*10 fmt1
      character*20 rstfnm

      data beta,hh0,gred,xe,xw,yn,ys/2.29e-11,150.,5.6e-02,156.,
     1  0.,29.75,-29.75/
ct80  NOTE:  rrd is the damping time (in years) for ocean dynamics
cB88       data ddt,rrd/10.,2.5/
cstable       data ddt,rrd/10.,.8/
cneutral       data ddt,rrd/10.,1.5/
cunstable       data ddt,rrd/10.,2.5/
coriginal      data ddt,rrd/10.,1./
      data ddt,rrd/10.,1.5/
      open(unit=11,file='out1',status='unknown')
      open(unit=9,file='time_a',status='old')
      open(unit=19,file='tfield1', status='unknown',
     1 form='formatted')
      open(unit=26,file='model.wind.ano1', status='unknown',
     1 form='formatted')
      open(unit=27,file='model.temp.ano1', status='unknown',
     1 form='formatted')
      open(unit=28,file='model.hfield.ano1', status='unknown',
     1 form='formatted')
c
c     set constants
c   note that the friction damping time is 1/2.5 years-1; reduced gravity
c    is 5.6e-02 m/s**2, and hh0 is 150 m. therefore an equivalent depth of
c    app 86 cm is achieved
c
      iseed=9568743
ct80  NOTE:  rhocd is the drag coefficient for wind stress on the ocean
cB88      rhocd=2.2e-03
cstable      rhocd=1.6e-03
c  Note:  neutral stability is somewhere between 2.8-2.9, for rrd = 1.5
cneut      rhocd=2.7e-03
cunstable      rhocd=2.2e-03
coriginal      rhocd=2.2e-03
      rhocd=2.0e-3
      rhoh20=1.026e+03
      nddt=ddt
      ddt=ddt*3600.*24.
      ddx=abs(xe-xw)*110000./float(m)
      ddy=abs(yn-ys)*110000./float(n)
      c=sqrt(gred*hh0)
      xl=sqrt(c/beta)
      xt=1./sqrt(c*beta)
      dt=ddt/xt
      dx=ddx/xl
      dy=ddy/xl
      rd=1./(rrd*360.*3600.*24.)
      rs=xt/(2.*24.*3600.)
      r=rd/xt
      alp=dt/dx
      a1=(1.-alp**2)/dt
      a2=(1.+alp**2)/dt
      a3=2.*alp/dt
      p=int(alp)
      gamma=alp-p
      aknew=0.
      write(11,900)xe,xw,yn,ys,ddx,ddy,xl,dx,dy,ddt,xt,dt,c
      write(11,920)m,n,p,alp,gamma
      cert=rhoh20*hh0*c/xt
      fert=exp(-rd*ddt)
      fert2=exp(-rd*ddt/2.)
      wndnon=rhocd/(fert*cert)
      wndno2=rhocd/(fert2*cert)

      write(11,930) fert,cert,wndnon
      do 10 j=1,n
  10  y(j)=(j-(n+1)/2.)*dy
      call setphi
      call zeroak
c  call setup here
      call setup(alp)
c  read in mean winds
      call readuo
      call ammset
cddd
cDV071007 do this later, under npohfx     call setuam
c3yr      call setuam
c      call start
c   ntime is the length of integration in ddt units,
c   nstart is the day to start at in days units,
c   nrst is the time to write restart tape (in nddt units)and nrstfl
c   is the flag to read restart tape (nrstfl=1, read restart tape)
c   if nadj = 1, do the adjoint calculation
c   nstyr is the year of the restart file name (see below)
c   npoamp is the amplification of the input data, or noise amplitude
      read(9,*)ntime,nstart,nrst,nrstfl,nadj,npoamp
      write(11,967)ntime,nstart,nrst,nrstfl,nadj,nstyr,npoamp
  967 format(2x,'ntime nstart nrst nrstfl nadj nstyr npoamp',
     &     /,2x,6i9,e12.4)
      nrst=nrst*nddt
c  Do this after reading the restart - if ntime < 0 from time_a, then
c  define ntime = nstart-ntime*ddt
c      ntime=ntime*nddt
      n2rst=2*nrst
c
c3yr  Now, get external forcing
c  We've rewritten npohfx to load in the som2x winds
      call npohfx(npoamp)
c
c  figure degrees of freedom
c
      if(nadj.eq.0)then
         ndeg=1
      else
c         ndeg= (1 + nmodes/2)  * 79 + 26*12 + 1
         ndeg = 26*12 + 1
      endif

      print *,'ndeg =', ndeg
      indout = 0

c
c  Initialize random number generator
c      call random_seed(generator = 2)
c      iseed(1) = 295814395
c      iseed(2) = 918347523

      do 700   iadjoin=1,ndeg
c      do 700  iadjoin=nndeg,ndeg

      if(nrstfl.eq.0) go to 703
cDV071007      call restart(nstart,nrstfl)
c
c  Determine restart file name
      write(fmt1,fmt='(a,i1,a)') '(a,i',
     &     int(log10(real(nstyr+0.5)))+1,')'
      write(rstfnm,fmt1) 'Restart/restart.',nstyr
      call restart2(nstart,rstfnm,iseed)

c
c     if we are in adjoint mode, get the perturbation and add to the
c     variables
c
      if(nadj.eq.1) call perterb(iadjoin,nstart)

 703  continue
c
c  Initialize random number generator AFTER reading iseed from restart file
c      call random_seed(put = iseed(1:2))
      call random_seed(iseed)
     
      if (ntime.ge.0) then
         ntime=ntime*nddt
      else
         ntime=nstart-ntime*nddt
      endif
      nstart=nstart+nddt
      
ctemp
c      go to 700
ctemp

c      write(11,923)nstart
      nday=mod((nstart-1),360)+1
      call initm(nday,mon0u,mon1u,mon0v,mon1v)
      mon0u2 = mon0u
      mon1u2 = mon1u

      do 200 ijk=nstart,ntime,nddt

c  This refers to the simulations with 18mo of forcing... Ignore for this version.
c  Originally, this called clock (12mo), then inter.  We're going to
c  call clock2 (48mo), then clock (12mo), then inter2, and assign 
c  different mon0 and mon1 to each call to clock and clock2
      call clock(mod((ijk-1),360)+1,mon0u,mon1u,uinc)
      write(11,945)ijk
 945  format(' ntime = ',i8)
      call inter(mon0u,mon1u,uinc,ijk,nstart)
c      call clock2(mod((ijk-1),1440)+1,mon0u2,mon1u2,uinc)
c      call clock(mod((ijk-1),360)+1,mon0u,mon1u,uinc)
c      write(11,945)ijk
c 945  format(' ntime = ',i8)
c      call inter2(mon0u2,mon1u2,uinc,ijk,nstart)

      call nondim

      do 210 i=1,m
 210  call projef(i,fk(i))
      call charac(aknew)
      call rossby
c      print *,'done in rossby'
 954  format(2x,20(10e12.4,/))
c      call fixros
      call westbc(aknew)
      call switch(aknew)
c      do 300 i=1,20
c 300  write(10,954)(u(i,j),j=1,n)
      call printe(ijk,c,hh0)
      call thermo(ijk,iseed)
      call atmos(ijk)
c      if(mod(ijk,30).eq.0)write(26)ijk,uanom,vanom
      if(.true.) then
      if(mod(ijk,30).eq.0) then
        do 304 j=1,30
 304    write(26,972)(uanom(i,j),i=1,26)
        do 305 j=1,30
 305    write(26,972)(vanom(i,j),i=1,26)
      endif
      endif


c 973  if(mod(ijk,nrst).eq.0)call wrstart(ijk,n2rst)
 973  if((nrst.ne.0).and.(mod(ijk-300,nrst).eq.0))call wrstart2(ijk)

c  set un and v and hn to u v and h ; same with al the forcing
c   fields.
c
c  calculate new forcing fields by getting new ua, va all nondimensionallizing
c   them in nondim and away we go
c
c      write(10,940)ijk,(ak(j),j=1,mp)

c       if(ijk.ge.160)write(75)ak

 200  continue

c
c      write the endstate fields w/ no perturbations
c

          call writeyb(ijk-nddt,nadj)


 700  continue

 900  format('   xe     xw      yn      ys',/,4f7.2,//,
     1  '      ddx       ddy          xl          dx           dy',/,
     2  5e12.4,//,'     ddt       xt          dt          c',/,
     3  4e12.4)
 901  format(' ntime =',i4,/,16(13(f6.3,1x),/))
 910  format(20(2x,10e12.4,/))
 914  format(' the non dimensional f')
 920  format(2x,'m= ',i4,'  n= ',i4,'  p= ',i3,'  alp= ',f6.3,
     1 ' gamma= ',f5.3)
 921  format(' nyear  nday  mon0u  mon1u    uinc',
     2 '   mon0v  mon1v    vinc',/,
     1 i5,6x,i3,3x,i2,4x,i2,4x,f5.4,
     3 5x,i2,3x,i2,6x,f5.4)
 922  format(' ak(i)  ',/,20(2x,10e12.4,/))
c 923  format(' the integration starts at day (',i7,'-10)*10')
 930  format (2x,'fert=',e12.4,'   cert=',e12.4,' wndnon= ',
     1   e12.4)
 940  format (2x,'time= ',i7,' ak =',/,20(2x,10e12.4,/))
 961  format(i4,i4,i4,i4)
 972  format(30e12.4)
      close(9)
      close(11)
c      close(13)
c      close(14)
c      close(15)
c      close(16)
      close(19)
      close(20)
      close(21)
      close(26)
      close(27)
      close(28)
c      call finish
      stop
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine setphi
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      subroutine setphi
c
c   this routine calculates the funcion phi in Cane and Patton
c   and checks the normalization by caluclating the integral
c   of phi**2 from ys to yn
c
      parameter(m=78,mp=79,n=119,np=120)
      common/p1/phi(np)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/h1/npp,npm
      npp=np/2+1
      npm=npp-1
      write(11,914)
c      arg=-(dy/2.)**2/2.
c      phi(npm)=exp(arg)
      phi(npm)=1.
      do 10 j=npp,n
  10  phi(j)=phi(j-1)*(1.-y(j-1)*dy/2.)/(1.+
     1   y(j)*dy/2.)
      phi(np)=phi(n)*(1.-y(n)*dy/2.)/(1.+
     1   (y(n)+dy)*dy/2.)
      do 15 j=npm,2,-1
  15  phi(j-1)=phi(j)*(1.+y(j)*dy/2.)/(1.-
     1  y(j-1)*dy/2.)

c      write(11,915) (phi(i),i=1,np)
      sum=0.
      do 20 i=1,np
  20  sum=sum+phi(i)**2*dy
      sum=sqrt(sum)
      write(11,910)sum
      do 30 i=1,np
  30  phi(i)=phi(i)/sum
c      write(11,915) (phi(i),i=1,np)
      sum=0.
      do 40 i=1,np
  40  sum=sum+phi(i)
      cc=sum*dy
  910 format(' the squarte root of the integral is evalutated to',
     1 2x,e12.4)
  914  format(' in setphi')
  915  format('phi',20(2x,10e12.4,/))
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine zeroak
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine zeroak
c
c   this routine initializes ocean variables (e.g. Kelvin amplitude, 
c   ocean currents) to 0.
c
      parameter(m=78,mp=79,n=119,np=120)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m5/u(mp,n),v(m,np),h(mp,n)
      integer p
      do 10 i=1,mp
  10  ak(i)=0.
      do 20 i=1,mp
      do 20 j=1,m
      u(i,j)=0.
  20  h(i,j)=0.
      do 30 i=1,m
      do 30 j=1,np
  30  v(i,j)=0.
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine setup
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

       subroutine  setup(alp)
c
c  this subroutine sets up the array L used in C+P eqn 4.7
c   yim=(i-(np+1)/2.*dy-dy/2.  yip=yim+dy
c
c
       parameter(m=78,mp=79,n=119,np=120)
       double precision d
       common/so/d(n)
       common/m1/dx,dy,cc,a1,a2,a3,y(n)
       common/s10/us(26,30),vs(26,30),hs(26,30),rs,yy(30),cof(n)
       common/m3/xl(3,np),g(np)
       do 10 i=2,n
       im=i-1
       xl(1,i)=y(im)**2-4./dy**2
       xl(2,i)=y(im)**2+y(i)**2+8./dy**2+4.*alp
  10   xl(3,i)=y(i)**2- 4./dy**2

c   get d(i)
       d(1)=0.
       do 20 i=2,n
   20  d(i)=xl(3,i)/(xl(2,i)-xl(1,i)*d(i-1))

      do 200 j=1,n
c      yy(j)=y(4*j-2)
  200 cof(j)=100./(50.*(rs**2+y(j)**2))
c      write(11,962)rs,yy,cof
  962 format(' rs = ',e12.4,'  yy(j) =',/,3(10(e12.4,2x),/),/,
     1 'cof(j)=',/,8(10(e12.4,2x),/))
       return
       end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine readuo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine readuo
c
c   read in mean state for the ocean and atmosphere
c
      parameter(nx=26,nxn=25,ny=30,nyn=29)
cc add to readu; make sure that none of the common variables or the
cc varibles listed here appear in readu
      common/tem2/ubar(26,30,12),vbar(26,30,12),wbar(26,30,12),
     1 wave(26,30),tbar(26,30,12)
      common/pr/uave(26,30),h(26,30),vave(26,30),u(26,30),
     1 have(26,30),v(26,30)
      common /coup2/ to(26,30),tave(26,30)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)
c3yr      common/wnd2/uam(26,30,12),vam(26,30,12)
      common/wnd2/uam(26,30,12),vam(26,30,12)
      common/wnd1/uabar(26,30),vabar(26,30)
      dimension tz(26)
      character plabel*10,t*80,plabel2(20)*4
      data tz/10*2.,1.7,1.4,1.6,2.2,2.75,3.3,3.8,4.3,4.9,7*5.5/

      do 5 i=1,26
  5   tza(i)=tz(i)/100.
c  uave,vave and have are used temporarily to read in the mean
c  fields
       open(unit=17,file=
     1 './Data/ocean2b.annave.ascii',
     1   status='old',form='formatted')
      open(unit=18,file=
     1 './Data/sst.mean.SOM_fixedang_000_EC00.12mo',
     1 status='old',form='formatted')

      do 10 k=1,12
      read(17,924)plabel,uave,vave,wave,have
      read(18,917)plabel2,have
      do 20 i=1,26
      do 20 j=1,30
      ubar(i,j,k)=uave(i,j)
      vbar(i,j,k)=vave(i,j)
ccdd
           if(i.eq.26)then
           wbar(i,j,k)=wave(i,j)*1.5
           else
           wbar(i,j,k)=wave(i,j)
           endif
ccdd
      tbar(i,j,k)=have(i,j)
  20  continue
c      if(k.eq.1)write(11,900)plabel
c      if(k.eq.1)write(11,910)plabel2
  10  continue
      close(17)
      close(18)
 917  format(2x,20a4,/,26(6(5(e14.6,2x)/)//))
c 917  format(2x,20a4,/,30(26(e14.6,2x)/))
 924  format(2x,a10,/,4(26(6(5(e14.6,2x)/)//)))
c 924  format(2x,a10,/,4(30(26(e14.6,2x)/)//))

c      write(11,920)tza,hbar
      do 24 i=1,26
      do 24 j=1,30
  24  to(i,j)=0.

c 900  format(' the average currents are read in and stored - ',a10)
cc 910  format(' the average temperat are read in and stored - ',/,
cc     1 20a4)
c  910 format(2x,20a4)
c 920  format('the average thermal gradient',2(13(f6.4,1x),/),
c     1  'the average upper layer depth',2(13(f6.2,1x),/))


       open(unit=24,file=
     1 './Data/uv992sig.mean.SOM_fixedang_000_EC00.12mo',
     1 status='old', form='formatted')
      
      open(unit=36,file='model.wind.mean1', status='unknown',
     1 form='formatted')
c3yr      do 100 k=1,12
      do 107 k=1,12
      read(24,925)t,uabar,vabar
c      if(k.lt.4)write(11,930)t
c  930 format(a80)
      do 100 i=1,26
      do 100 j=1,30
ctest      write(*,*) vabar(i,j),uabar(i,j)
      vam(i,j,k)=vabar(i,j)
      uam(i,j,k)=uabar(i,j)
 100  continue
      
c Write out mean winds as a check
      do 105 j=1,30
 105  write(36,927)(uabar(i,j),i=1,26)
      do 106 j=1,30
 106  write(36,927)(vabar(i,j),i=1,26)

 107  continue

      close(24)
      close(36)
 925  format(2x,a80,/,2(26(6(5(e14.6,2x)/)//)))
 927  format(30e12.4)

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine ammset
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine ammset
c
c  initialize atmospheric quantities
c
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex a,cz,qf,vf,uf,g,z,c1
      common/ag/g(np),vf(np),uf(np),qf(np)
      common/ah/dyn,ep,c1,aaa,xl,e2
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/ad/u(mp,np),v(mp,np)
      common/coup/ua(26,30),va(26,30)
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      call zeroua(q,mp,np)
      call zeroua(qo,mp,np)
c do this before the reset tape is read
      call zeroua(uanm,26,30)
      call zeroua(vanm,26,30)
      call zeroua(qonm,26,30)
      call zeroua(divnm,26,30)
      call zeroua(ua,26,30)
      call zeroua(va,26,30)
      call atmset
      call atfour
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine zeroua
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine zeroua(u,mp,np)
      dimension u(mp,np)
      do 10 i=1,mp
      do 10 j=1,np
  10  u(i,j)=0.
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine atmset
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine atmset
c this subroutine sets all the atmospheric parameters and needs only
c to be called once before and atmospheric time steps
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex czero,c1,g1, d,chi,czerod
      common /aad/czerod
      common/adim/dx,dy
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /aa/czero
      common/ah/dyn,ep,c1,aaa,xl,e2
      common /e1/chi(np,kp),d(np,kp),g1(kp)
      common/ai/y(np)
      czero=cmplx(0.,0.)
      czerod=cmplx(0.,0.)
      c1=cmplx(0.,1.)
c  qbeta=the efficency of atmospheric feedback
c  divcon is the divergence value difference to halt the convergence loop
c  dx,dy are the dimensional atmospheric length scales
c  ca is the atmospheric phase speed
corig      qbeta=.750
cuncoup      qbeta=0.0
cunstable      qbeta=1.0
      qbeta=0.75
      divcon=1.e-06
      dy=2.*1.1e5
      dx=6.*1.1e5
      ca=60.
      beta=2.3e-11
      xl=sqrt(ca/(2.*beta))
      xt=1./sqrt(2.*ca*beta)
      dyn=dy/xl
      dxn=dx/xl
      epd=1.57607e-5
      ep=epd*xt
      aaa=6.3e6
      e2=ep*ep
      centy=np/2.+.5
      do 10 i=1,np
   10 y(i)=(i-centy)*dyn
c set up the arrays for the guassian elimination scheme
c the chi and d arrays never change. so they are set here
c for efficiency. See Holton's notes.
      do 20 i=1,kp
      a=(dyn**2)*(i-1)*xl/(aaa*2.*ep)
      g1(i)=cmplx(0.,a)
      do 20 j=1,np
      chi(j,i)=g1(i)-dyn*dyn*(e2+((i-1)*xl/aaa)**2)-2.-
     1  (dyn*y(j)/2.)**2
      if(j.eq.1)then
      d(j,i)=czero
      else
      d(j,i)=1./(chi(j,i)-d(j-1,i))
      endif
  20  continue
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine atfour
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine atfour
c set up the sin,cos arrays for the zonal fft.  c=(cos, -sin) and
c cz=conjg(c).  This setup assumes that grid point x=1 is at -pi.
c (note, grid x=1 is at 127E)
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex z,cz
      real*4 array(2),retval
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
c      retval=etime(array)
      pi=atan2(0.,-1.)
      dx=2.*pi/float(mp-1)
      do 10 k=1,kp
      xk=float(k-1)
      do 10 i=1,mp
      arg1=((i-1.5)*dx-pi)*xk
      arg2=((i-.5)*dx-pi)*xk
      b1=cos(arg1)+cos(arg2)
      b2=-(sin(arg1)+sin(arg2))
      z(i,k)=cmplx(b1,b2)*dx/2.
      cz(i,k)=cmplx(b1,-b2)/2.
      if(k.eq.1)then
      z(i,k)=z(i,k)/(2.*pi)
      else
      z(i,k)=z(i,k)/pi
      endif
c      write(*,*), etime(array)-retval
  10  continue
c     print *, etime(array)-retval
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine npohfx
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine npohfx(npoamp)
c
c  sets the initial condition (in this case an initial impulse in the shape
c  of a kelvin wave) for the model
c  NOTE:  IN this case, we've changed this to load in the som2x wind anomaleis
c      common/coup/uanom(26,30),vanom(26,30)
      common/coup3/fsns(26,30,12),uinit(26,30,12),vinit(26,30,12),
     1  fsnsf(26,30)
      common/wnd1/uabar(26,30),vabar(26,30)
      common/bigdom/fsnsb(60,30,12),fsnsbf(60,30),tbig(60,30),
     1  tbarb(60,30,12),tbarbf(60,30)
      real npoamp
cJC      open(unit=25,file=
cJC     1 './Data/FSNS.anom.SOM_fixedang_000_EC04mEC00.12mo',
cJC     1 status='old', form='formatted')
      open(unit=35,file=
     1 './Data/uv992sig.anom.SOM_fixedang_000_EC04mEC00.12mo',
     1 status='old', form='formatted')
      open(unit=37,file='model.wind.pert1', status='unknown',
     1 form='formatted')
      open(unit=38,
     1 file='./Data/FSNS.globanm.SOM_fixedang_000_EC04mEC00.12mo',
     1 form='formatted')
      do 107 k=1,12
cJC      read(25,926)t,uabar
      do 100 i=1,26
      do 100 j=1,30
      fsnsf(i,j)=0.
 100  fsns(i,j,k)=0.
cJC      fsnsf(i,j)=uabar(i,j)
cJC 100  fsns(i,j,k)=uabar(i,j)

      read(38,938)t,fsnsbf,tbarbf
      do 101 i=1,60
      do 101 j=1,30
      fsnsb(i,j,k)=0.
      tbarb(i,j,k)=tbarbf(i,j)
      tbig(i,j)=0.
      tbarbf(i,j)=0.
 101  fsnsbf(i,j)=0. 
      read(35,928)t,uabar,vabar
      do 102 i=1,26
      do 102 j=1,30
      uinit(i,j,k)=uabar(i,j)*npoamp
 102  vinit(i,j,k)=vabar(i,j)*npoamp
c Write out wind perterbation as a check
      do 105 j=1,30
 105  write(37,927)(fsnsf(i,j),i=1,26)
 
 107  continue
      close(25)
      close(35)
      close(37)
      close(38) 
c 926  format(2x,a80,/,2(26(6(5(e14.6,2x)/)//)))
 926  format(2x,a80,/,(26(6(5(e14.6,2x)/)//)))
 928  format(2x,a80,/,2(26(6(5(e14.6,2x)/)//)))
 927  format(30e12.4)
 938  format(2x,a80,/,2(60(6(5(e14.6,2x)/)//)))
 
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine setuam
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c      subroutine setuam
c
c  sets the initial condition (in this case an initial impulse in the shape
c  of a kelvin wave) for the model
c
c      common/coup/uanom(26,30),vanom(26,30)
c      common/coup3/uinit(26,30),vinit(26,30)
c      call zeroua(uinit,26,30)
c      do 10 k=1,12
c      do 10 j=1,30
c      xlat=exp(-(((j-16)*2+1.)**2/40.))
c      do 10 i=4,11
ctest      write(*,*)i,j,xlat
c  10  uinit(i,j,k)=2.*xlat
c      return
c      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine setumm
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c      subroutine setumm
c
c   Sets up a different initial condition... this is not called.
c
c      common/coup/uanom(26,30),vanom(26,30)
c      common/coup3/uinit(26,30)
c      call zeroua(uanom,26,30)
c      call zeroua(vanom,26,30)
c      do 10 j=1,30
c      xlat=exp(-(((j-16)*2+1.)**2/40.))
c      do 10 i=4,11
c  10  uanom(i,j)=2.*xlat
c      return
c      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine restart2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine restart2(nstart,rstfnm,iseed)
c
c  read in the restart file
c
      parameter(m=78,mp=79,n=119,np=120)
      parameter(nnp=70,kp=20,mmp=61,mpm=60)
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common /af/ z(mmp,kp),q(mmp,nnp),cz(mmp,kp),qo(mmp,nnp)

      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /coup2/ to(26,30),tave(26,30)
      common/coup/uanom(26,30),vanom(26,30)
      common /rannum/ rnum3(28,34)
      complex a,cz,qf,vf,uf,g,z,c1
      integer p,iseed
      character*34 label
      character*20 rstfnm

      data label/' The temperature field at restart'/
c      rstfnm = 'restart.1'
      open (unit=76,file=rstfnm, status='old',form='unformatted')
      read(76)nstart,iseed,ak,u,v,h,to,uanom,vanom,
     &     divmn,divnm,qonm,q,div,rnum3
 
c      read(76)nstart,ak,qros,to,uanom,vanom,divmn,divnm,qonm,q
 
c      write(11,900)nstart
 
c 900  format(' a restart is enabled at time',i6)
       close(76)
 
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine restart
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine restart(nstart,nrstfl)
c
c  read in the restart file
c
      parameter(m=78,mp=79,n=119,np=120)
      parameter(nnp=70,kp=20,mmp=61,mpm=60)
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common /af/ z(mmp,kp),q(mmp,nnp),cz(mmp,kp),qo(mmp,nnp)

      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /coup2/ to(26,30),tave(26,30)
      common/coup/uanom(26,30),vanom(26,30)
      complex a,cz,qf,vf,uf,g,z,c1
      integer p
      character*34 label
      data label/' The temperature field at restart'/
      open (unit=76,file='restart', status='old',form='unformatted')
      read(76)nstart,ak,u,v,h,to,uanom,vanom,divmn,divnm,qonm,q,div
 
c      read(76)nstart,ak,qros,to,uanom,vanom,divmn,divnm,qonm,q
 
c      write(11,900)nstart
 
c 900  format(' a restart is enabled at time',i6)
       close(76)
 
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine perterb
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine perterb(iadjoin,nstart)
c
c  if the model is being run in adjoint mode, this routine perturbs each
c  variable accordingly 
c
      parameter(m=78,mp=79,n=119,np=120)
c      parameter(nx=26,nxn=25,ny=30,nyn=29)
      parameter(nmodes=6)
      common /coup2/ to(26,30),tave(26,30)
      common/qros1/qros(mp,nmodes),qrosp(mp,nmodes)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      integer p

c      epsilq=.001
      epsilt=.03
c      epsilt=.06
c      epsilt=.015

      print *,'adding ssta perturbation mode =',iadjoin

      nsst=26*12

c     now sst only

c  note last case is the control run:  no perturbation

      if(iadjoin.gt.nsst)go to 15
      imode=(iadjoin-1)/26 + 10
      ii=mod(iadjoin-1,26)+1
      to(ii,imode)=to(ii,imode) + epsilt
      print *,'sst, (iadjoin,i,j) = ',iadjoin,imode,ii
c
c  now get consitent atmospheric fields to go with this
c
      nday=mod((nstart-1),360)+1
      call initm(nday,mon0u,mon1u,mon0v,mon1v)
      call clock(mod((nstart-1),360)+1,mon0u,mon1u,uinc)
cJC NOTE: the call below needs to be fixed if this subroutine is to be used (needs one more input variable before nstart). Currently it is not used.  
      call inter(mon0u,mon1u,uinc,nstart)
      call atmos(nstart)
c      print *,' getting new wind, nstart, nday, mon0u, mon1u,', 
c     1  nstart, nday, mon0u, mon1u

      print *,' getting new wind, nstart, nday, mon0u, mon1u, uinc,', 
     1  nstart, nday, mon0u, mon1u, uinc

c      print *,' mon0v mon1v, uinc ',mon0v, mon1v, uinc


 15   continue
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine initm
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine initm(ndayst,mon0u,mon1u,mon0v,mon1v)
c
c   set time coordinates
c
      common /cl2/mthend(12),day(12)
      do 10 i=1,12
  10  if(ndayst.le.mthend(i)) go to 20
  20  if((i.eq.1).or.(i.eq.13)) then
      mon0u=12
      mon1u=1
      else
      mon1u=i
      mon0u=mon1u-1
      endif
      if(ndayst.gt.5)then
      ndayv=ndayst-5
      else
      ndayv=ndayst+360-5
      endif
      do 15 i=1,12
  15  if(ndayv.le.mthend(i)) go to 25
  25  if((i.eq.1).or.(i.eq.13)) then
      mon0v=12
      mon1v=1
      else
      mon1v=i
      mon0v=mon1v-1
      endif
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine clock
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine clock(nday,mon0,mon1,dt)
c this subroutine takes the julian day nday and figures out
c which months to interpolate from and what the weighting
c should be on the intgerpolation
c  e.g.    january 15 falls in the middle of january
c     so the wind would be = december + 1.*(jan-dec)
c       f(x)= f(previous month)+ dt*(f(next mon)-f(prev mon))
      common /cl2/mthend(12),day(12)
      if(nday.ge.mthend(mon1)) then
      if((mon0.eq.12).and.(nday.gt.mthend(12))) go to 10
      mon0=mon1
      mon1=mon1+1
      if(mon1.eq.13)mon1=1
      else
      endif
  10  if(mon1.eq.1) then
      dt= float(nday-mthend(12))/day(1)
      if(dt.lt.0.) dt=1.-float(mthend(1)-nday)/day(mon1)
      else
      dt=float(nday-mthend(mon0))/day(mon1)
      endif
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine clock2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine clock2(nday,mon0,mon1,dt)
c this subroutine takes the julian day nday and figures out
c which months to interpolate from and what the weighting
c should be on the intgerpolation
c  NOTE:  this is for the 48 month wind file
c  e.g.    january 15 falls in the middle of january
c     so the wind would be = december + 1.*(jan-dec)
c       f(x)= f(previous month)+ dt*(f(next mon)-f(prev mon))
      common /cl3/mthend3(48),day3(48)
c3yr      common /cl2/mthend(12),day(12)
      if(nday.ge.mthend3(mon1)) then
      if((mon0.eq.48).and.(nday.gt.mthend3(48))) go to 10
      mon0=mon1
      mon1=mon1+1
      if(mon1.eq.49)mon1=1
      else
      endif
  10  if(mon1.eq.1) then
      dt= float(nday-mthend3(48))/day3(1)
      if(dt.lt.0.) dt=1.-float(mthend3(1)-nday)/day3(mon1)
      else
      dt=float(nday-mthend3(mon0))/day3(mon1)
      endif
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine inter
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine inter(mon0,mon1,uinc,ntime,nstart)
c
c   interpolate to appropriate time between mon0 and mon1
c
      parameter(m=78,mp=79,n=119,np=120)
      common/o1/ua(m,n),va(mp,np),wndnon,wndno2,fert2
      common /coup2/ to(26,30),tave(26,30)
      common/coup/uanom(26,30),vanom(26,30)
      common/wnd1/uabar(26,30),vabar(26,30)
c3yr      common/wnd2/uam(26,30,12),vam(26,30,12)
      common/wnd2/uam(26,30,12),vam(26,30,12)
      common/coup3/fsns(26,30,12),uinit(26,30,12),vinit(26,30,12),
     1  fsnsf(26,30)
      common/bigdom/fsnsb(60,30,12),fsnsbf(60,30),tbig(60,30),
     1  tbarb(60,30,12),tbarbf(60,30)
      common/tauin/ temp(mp,np),um(26,30),vm(26,30)
c calculate the pseudo winds um,vm from the wind anomalies uanom,vanom
      do 100 i=1,26
      do 100 j=1,30
c  uabar and vabar are the average wind fields at the time of interest
      uabar(i,j)=uam(i,j,mon0)+uinc*(uam(i,j,mon1)-uam(i,j,mon0))
      vabar(i,j)=vam(i,j,mon0)+uinc*(vam(i,j,mon1)-vam(i,j,mon0))
c      xu=uabar(i,j)
c      xv=vabar(i,j)
c      xu=uabar(i,j)+uanom(i,j)
c      xv=vabar(i,j)+vanom(i,j)
c23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567
c Get forcing for this time step
      fsnsf(i,j)=
     1  fsns(i,j,mon0)+uinc*(fsns(i,j,mon1)-fsns(i,j,mon0))
      xu=uabar(i,j)+uanom(i,j)+
c      xu=uabar(i,j)+
     & uinit(i,j,mon0)+uinc*(uinit(i,j,mon1)-uinit(i,j,mon0))
cZonalWindOnly - only include vabar + vanom --> remove second line
      xv=vabar(i,j)+vanom(i,j)+
c      xv=vabar(i,j)+
     & vinit(i,j,mon0)+uinc*(vinit(i,j,mon1)-vinit(i,j,mon0))
c ....      if(ntime.le.120) xu=xu+uinit(i,j)
c      if(ntime.le.120) xu=xu+uinit(i,j)

cx      if(ntime.le.1200) xu=xu+uinit(i,j)
cx      if(ntime.gt.1200) stop

c      if(ntime.le.120) xu=xu+uinit(i,j)
c      if ((ntime-nstart).le.150) then
c         to(i,j)=to(i,j)+uinit(i,j)
c         write(*,*) 'hello',i,j,ntime,nstart,ntime-nstart,uinit(i,j)
c      endif

      uarab=sqrt(uabar(i,j)**2+vabar(i,j)**2)
      uabs=sqrt(xu**2+xv**2)
c
c  Add WES feedback
c      write(*,*) 0.04*(uabs-uarab)
c      to(i,j) = to(i,j) + 0.6*0.05*(uabs-uarab)

      tu=uabs*xu-uarab*uabar(i,j)
      tv=uabs*xv-uarab*vabar(i,j)
c      write(*,*) tu,tv
      if (tu.eq.0. .and. tv.eq.0.)   then
      alp=0.
      else
      alp=atan2(tv,tu)
      endif
      velo=(tu**2+tv**2)**0.25
      um(i,j)=velo*cos(alp)
  100 vm(i,j)=velo*sin(alp)
c  Add fsnsbf
      do 101 i=1,60
      do 101 j=1,30
      fsnsbf(i,j)=
     1  fsnsb(i,j,mon0)+uinc*(fsnsb(i,j,mon1)-fsnsb(i,j,mon0))
 101  tbarbf(i,j)=
     1  tbarb(i,j,mon0)+uinc*(tbarb(i,j,mon1)-tbarb(i,j,mon0))
c
c  write out total psuedo wind stress
c

c      if(mod(ntime,30).eq.0) write(43)ntime,um,vm


      call tau(ntime)
      do 200 j=1,30
      temp(1,j)=um(1,j)
      temp(2,j)=um(1,j)
      temp(m,j)=um(26,j)
      temp(m-1,j)=um(26,j)
      do 200 i=3,m-2
      isx=i/3
      xinc=(mod(i,3)+1.)/3.
 200  temp(i,j)=um(isx,j)+xinc*(um(isx+1,j)-um(isx,j))

      do 210 i=1,m
      ua(i,1)=temp(i,1)
      ua(i,2)=temp(i,1)
      ua(i,n)=temp(i,30)
      ua(i,n-1)=temp(i,30)
      do 210 j=3,n-2
      yinc=(mod(j+1,4)+1.)/4.
      isy=(j+1)/4
 210  ua(i,j)=temp(i,isy)+yinc*(temp(i,isy+1)-temp(i,isy))

      do 220 j=1,30
      temp(1,j)=vm(1,j)
      temp(2,j)=vm(1,j)
      temp(mp,j)=vm(26,j)
      temp(m,j)=vm(26,j)
      do 220 i=3,m-1
      isx=i/3
      xinc=(mod(i,3)+.5)/3.
 220  temp(i,j)=vm(isx,j)+xinc*(vm(isx+1,j)-vm(isx,j))

      do 230 i=1,mp
      va(i,1)=temp(i,1)
      va(i,2)=temp(i,1)
      va(i,np)=temp(i,30)
      va(i,n)=temp(i,30)
      do 230 j=3,n-1
      yinc=(mod(j+1,4)+.5)/4.
      isy=(j+1)/4
 230  va(i,j)=temp(i,isy)+yinc*(temp(i,isy+1)-temp(i,isy))
      mm=1
      nn=10
c      write(11,901)mon0v
c      do 300 j=1,30
c 300  write(11,900)(vam(i,j,mon0u),i=mm,nn)
c      write(11,901)mon1v
c      do 310 j=1,30
c 310  write(11,900)(vam(i,j,mon1u),i=mm,nn)
c      write(11,903)
c      do 320 j=1,30
c 320  write(11,900)j,(um(i,j),i=mm,nn)
c      write(11,904)
c      do 330 j=1,n
c 330  write(11,900)j,(ua(i,j),i=mm,nn)
c      write(11,905)
c      do 340 j=1,n
c 340  write(11,900)(ua(i,j),i=m-9,m)
 900  format(i3,10(f7.3,3x))
c 901  format(//,' the u winds at month ',i2)
 903  format(//,' the time average  u winds ')
 904  format(//,' the model winds at i=1,10')
 905  format(//,' the model winds at i=m-9,m')
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine inter2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine inter2(mon0,mon1,uinc,ntime,nstart)
c
c   interpolate to appropriate time between mon0 and mon1
c
      parameter(m=78,mp=79,n=119,np=120)
      common/o1/ua(m,n),va(mp,np),wndnon,wndno2,fert2
      common /coup2/ to(26,30),tave(26,30)
      common/coup/uanom(26,30),vanom(26,30)
      common/wnd1/uabar(26,30),vabar(26,30)
c3yr      common/wnd2/uam(26,30,12),vam(26,30,12)
      common/wnd2/uam(26,30,12),vam(26,30,12)
      common/coup3/fsns(26,30,12),uinit(26,30,12),vinit(26,30,12),
     1  fsnsf(26,30)
      common/tauin/ temp(mp,np),um(26,30),vm(26,30)
c calculate the pseudo winds um,vm from the wind anomalies uanom,vanom
c      write(*,*) 'hello',mon0,mon1,uinc,ntim,nstart
      do 100 i=1,26
      do 100 j=1,30
c  uabar and vabar are the average wind fields at the time of interest
      uabar(i,j)=uam(i,j,mon0)+uinc*(uam(i,j,mon1)-uam(i,j,mon0))
      vabar(i,j)=vam(i,j,mon0)+uinc*(vam(i,j,mon1)-vam(i,j,mon0))
cXXXX  Need to interpolate som2x wind anomaly using the same formula
cXXXX  as above.  Just add it to the two lines below, to avoid introducing
cXXXX  another variable.
c      write(*,*) ntime,i,j,uabar(i,j),vabar(i,j),uanom(i,j),vanom(i,j),
c     &  uinit(i,j,mon1),vinit(i,j,mon1),mon1,mon0
      if (ntime.lt.1440) then
      xu=uabar(i,j)+uanom(i,j)+
     & uinit(i,j,mon0)+uinc*(uinit(i,j,mon1)-uinit(i,j,mon0))
      xv=vabar(i,j)+vanom(i,j)+
     & vinit(i,j,mon0)+uinc*(vinit(i,j,mon1)-vinit(i,j,mon0))
      else
      xu=uabar(i,j)+uanom(i,j)
      xv=vabar(i,j)+vanom(i,j)
      endif
c      write(*,*) mon0,mon1,i,j,xu,xv
c ....      if(ntime.le.120) xu=xu+uinit(i,j)
c      if(ntime.le.120) xu=xu+uinit(i,j)

cx      if(ntime.le.1200) xu=xu+uinit(i,j)
cx      if(ntime.gt.1200) stop

c      if(ntime.le.120) xu=xu+uinit(i,j)
c      if ((ntime-nstart).le.150) then
c         to(i,j)=to(i,j)+uinit(i,j)
c         write(*,*) 'hello',i,j,ntime,nstart,ntime-nstart,uinit(i,j)
c      endif

      uarab=sqrt(uabar(i,j)**2+vabar(i,j)**2)
      uabs=sqrt(xu**2+xv**2)
c
c  Add WES feedback
c      write(*,*) 0.04*(uabs-uarab)
c      to(i,j) = to(i,j) + 0.6*0.05*(uabs-uarab)

      tu=uabs*xu-uarab*uabar(i,j)
      tv=uabs*xv-uarab*vabar(i,j)
c      write(*,*) tu,tv
      if (tu.eq.0. .and. tv.eq.0.)   then
      alp=0.
      else
      alp=atan2(tv,tu)
      endif
      velo=(tu**2+tv**2)**0.25
      um(i,j)=velo*cos(alp)
  100 vm(i,j)=velo*sin(alp)
c
c  write out total psuedo wind stress
c

c      if(mod(ntime,30).eq.0) write(43)ntime,um,vm


      call tau(ntime)
      do 200 j=1,30
      temp(1,j)=um(1,j)
      temp(2,j)=um(1,j)
      temp(m,j)=um(26,j)
      temp(m-1,j)=um(26,j)
      do 200 i=3,m-2
      isx=i/3
      xinc=(mod(i,3)+1.)/3.
 200  temp(i,j)=um(isx,j)+xinc*(um(isx+1,j)-um(isx,j))

      do 210 i=1,m
      ua(i,1)=temp(i,1)
      ua(i,2)=temp(i,1)
      ua(i,n)=temp(i,30)
      ua(i,n-1)=temp(i,30)
      do 210 j=3,n-2
      yinc=(mod(j+1,4)+1.)/4.
      isy=(j+1)/4
 210  ua(i,j)=temp(i,isy)+yinc*(temp(i,isy+1)-temp(i,isy))

      do 220 j=1,30
      temp(1,j)=vm(1,j)
      temp(2,j)=vm(1,j)
      temp(mp,j)=vm(26,j)
      temp(m,j)=vm(26,j)
      do 220 i=3,m-1
      isx=i/3
      xinc=(mod(i,3)+.5)/3.
 220  temp(i,j)=vm(isx,j)+xinc*(vm(isx+1,j)-vm(isx,j))

      do 230 i=1,mp
      va(i,1)=temp(i,1)
      va(i,2)=temp(i,1)
      va(i,np)=temp(i,30)
      va(i,n)=temp(i,30)
      do 230 j=3,n-1
      yinc=(mod(j+1,4)+.5)/4.
      isy=(j+1)/4
 230  va(i,j)=temp(i,isy)+yinc*(temp(i,isy+1)-temp(i,isy))
      mm=1
      nn=10
c      write(11,901)mon0v
c      do 300 j=1,30
c 300  write(11,900)(vam(i,j,mon0u),i=mm,nn)
c      write(11,901)mon1v
c      do 310 j=1,30
c 310  write(11,900)(vam(i,j,mon1u),i=mm,nn)
c      write(11,903)
c      do 320 j=1,30
c 320  write(11,900)j,(um(i,j),i=mm,nn)
c      write(11,904)
c      do 330 j=1,n
c 330  write(11,900)j,(ua(i,j),i=mm,nn)
c      write(11,905)
c      do 340 j=1,n
c 340  write(11,900)(ua(i,j),i=m-9,m)
 900  format(i3,10(f7.3,3x))
c 901  format(//,' the u winds at month ',i2)
 903  format(//,' the time average  u winds ')
 904  format(//,' the model winds at i=1,10')
 905  format(//,' the model winds at i=m-9,m')
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine tau , function st
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine tau(ntime)
c
c   This routine can be used to calculate wind stress averaged over two 
c   different areas.  It does not affect the model solution.
c
      parameter(m=78,mp=79,n=119,np=120)
      common/tauin/ temp(mp,np),um(26,30),vm(26,30)
c calculates tau1 and tau2 indexes in dynes/(cm**2)
      data rocd/3.2e-03/
      sum1=0.
      sum2=0.
      do 20 i=3,11
  20  sum1=sum1+(.5*(st(i,13)+st(i,18))+st(i,15)+st(i,16)+st(i,17)
     1 + st(i,14))/5.
      tau1=sum1*rocd*10./9.
      do 30 i=12,17
  30  sum2=sum2+(.5*(st(i,13)+st(i,18))+st(i,15)+st(i,16)+st(i,17)
     1 + st(i,14))/5.
      tau2=sum2*rocd*10./6.
c      if(mod(ntime,30).eq.0)      write(28)ntime,tau1,tau2
      return
      end


      function st(i,j)
      parameter(m=78,mp=79,n=119,np=120)
      common/tauin/ temp(mp,np),um(26,30),vm(26,30)
      st=um(i,j)*sqrt(um(i,j)**2+vm(i,j)**2)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine nondim , function corner
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine nondim
c
c   this routine nondimensionalizes the x forcing
c    note that this whole program assumes that the wind is known at
c    t=0(uwind) and t=n/2(vwind)
c
      parameter(m=78,mp=79,n=119,np=120)
      common/o1/ua(m,n),va(mp,np),wndnon,wndno2,fert2
      common/m2/f(m,n),fert
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      do 10 i=1,m
      ip=i+1
      do 10 j=1,n
      jp=j+1
      uabs=sqrt(ua(i,j)**2+(va(ip,j)+va(i,j)+va(ip,jp)
     1  +va(i,jp))**2/16.)
 10   f(i,j)=wndno2*ua(i,j)*uabs
      do 20 i=2,m
      im=i-1
      do 20 j=2,n
      jm=j-1
      uabs=sqrt(va(i,j)**2+(ua(im,jm)+ua(im,j)+ua(i,jm)+
     1   ua(i,j))**2/16.)
 20   gwn(i,j)=wndnon*va(i,j)*uabs
      do 30 j=2,n
      jm=j-1
      gwn(1,j)=wndnon*va(1,j)*sqrt(va(1,j)**2+(ua(1,jm)
     1  +ua(1,j))**2/4.)
  30  gwn(mp,j)=wndnon*va(mp,j)*sqrt(va(mp,j)**2+(ua(m,jm)
     1  +ua(m,j))**2/4.)
      do 40 i=2,m
      im=i-1
      gwn(i,1)=wndnon*va(i,1)*sqrt(va(i,1)**2+(ua(im,1)+
     1  ua(i,1))**2/4.)
  40  gwn(i,np)=wndnon*va(i,np)*sqrt(va(i,np)**2+(ua(im,n)
     1  +ua(i,n))**2/4.)
      gwn(1,1)=corner(va(1,1),ua(1,1),wndnon)
      gwn(mp,np)=corner(va(mp,np),ua(m,n),wndnon)
      gwn(1,np)=corner(va(1,np),ua(1,n),wndnon)
      gwn(mp,1)=corner(va(mp,1),ua(m,1),wndnon)
      return
      end

      function corner(a,b,wndnon)
      corner=wndnon*a*sqrt(a**2+b**2)
      return
      end


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine projef
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine projef(k,sum)
c
c    this routine projects the nondimensional x forcing onto the phi
c       note the x forcing (f) is centered on the equator, as is u.
c       the phi funtions are calculated in between the f's
c
      parameter(m=78,mp=79,n=119,np=120)
      common/p1/phi(np)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/m2/f(m,n),fert
      sum=0.
      do 10 i=1,n-1
  10  sum=sum+(2.*(f(k,i)*phi(i)+f(k,i+1)*phi(i+1))+ phi(i)*
     1  f(k,i+1)+phi(i+1)*f(k,i))
      sum=sum*dy/12.+dy*f(k,n)*(3.*phi(n)+phi(np))/16.
     2  +f(k,1)*dy*(3.*phi(1)+phi(np))/16.
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine charac
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine charac(aknew)
c
c   this routine calulates the kelvin wave response using
c   the method of charcteristics. the equation 4.3 of Cane and
c   Patton is solved:
c      d/dt (ak) + d/dx (ak) = fk
c   So
c    ak(n+1)=an(n,x-cnt) + forcing
c   aknew is the value of ak at (n+1,x=0) from the Rossby
c   solution. the values of ak at n+1 (akp) are returned to
c   the main program as ak.
c
      parameter(m=78,mp=79,n=119,np=120)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/c2/fk(m)
      common/m2/f(m,n),fert
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      integer p
c
c  aknew is the new time value of ak at x=0
c  note alp=ddt*c/ddx and is between 1 and 10 for stability
c  p is int(alp)

      do 9 i=1,mp
  9   akp(i)=0.
      do 10 i=2,mp
      xin=float(i)-alp
      in=i-alp
      if(in.ge.1) akp(i)=ak(in)+(ak(in+1)-ak(in))*(xin-in)
      sum=0
      if((i-p).lt.2)go to 20
      sum=gamma*fk(i-p-1)
  20  do 30 j=1,p
      jj=i-j
      if(jj.lt.1) go to 30
      sum=sum+fk(jj)
  30  continue
  10  akp(i)=akp(i)+dx*sum

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine rossby
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine rossby
c
c   I think this routine calculates the rossby wave amplitude via the 
c   method of characteristics, given an eastern boundary reflection
c
      parameter(m=78,mp=79,n=119,np=120)
      common/h1/npp,npm
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/c2/fk(m)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/m2/f(m,n),fert
      common/m3/xl(3,np),g(np)
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common/g1/gsum(n)
      common/p1/phi(np)
      dimension r1(n),r3(n)
      integer p
c   get eastern bc
      call eastbd
c
c      call fixros(mp)
      do 5 i=m,1,-1
      ip=i+1
c  set up coefficients for matrix
      do 10 j=1,n
      r1(j)=2.*(f(i,j)-(1.+alp)*fk(i)*phi(j))
     1  +a1*u(ip,j)-a2*(un(ip,j)-u(i,j))-a3*(hn(ip,j)-h(i,j))
  10  r3(j)=2.*(alp*f(i,j)-(1.+alp)*fk(i)*phi(j))
     1  +a1*h(ip,j)-a2*(hn(ip,j)-h(i,j))-a3*(un(ip,j)-u(i,j))
c
c  check the expresion for r3 since it is not symmetric with r1
c
      do 20 j=2,n
  20  g(j)=2.*a1*gwn(i,j)-(y(j-1)*r1(j-1)+y(j)*r1(j))-2.*(r3(j)
     1  -r3(j-1))/dy
c solve matrix
      call solve(i)

c  900 format('i= ',i5,/,3(11(10e12.4,/),9e12.4,//))
      do 30 j=1,n
      jp=j+1
      hn(i,j)=(r3(j)+v(i,j)*(2./dy+alp*y(j))+v(i,jp)*(alp*y(j)
     1  -2./dy))/a1
  30  un(i,j)=(r1(j)+v(i,j)*(y(j)+2.*alp/dy)+v(i,jp)*(y(j)-
     1  2.*alp/dy))/a1
      call fixros(i)
  5   continue
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine eastbd
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine eastbd
c
c     this routine calculates the eastern boundary values of
c     un and hn at all y
c
      parameter(m=78,mp=79,n=119,np=120)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/g1/gsum(n)
      common/p1/phi(np)
      integer p
      call hsum(h0)
      do 10 j=1,n
      un(mp,j)=-akp(mp)*phi(j)
  10  hn(mp,j)=h0+gsum(j)+un(mp,j)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine hsum
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine hsum(h0)
c
c   this subroutine calculates h0
c
      parameter(m=78,mp=79,n=119,np=120)
      common/h1/npp,npm
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/g1/gsum(n)
      integer p
      gsum(npm)=0.
      do 10 j=npp,n
  10  gsum(j)=dy*gwn(mp,j)+gsum(j-1)
      do 20 j=npm,2,-1
  20  gsum(j-1)=gsum(j)-dy*gwn(mp,j)
      call projeg(sum)
      h0=(2.*akp(mp)-sum)/cc
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine projeg
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine projeg(sum)
c
c    this routine projects the nondimensional x forcing onto the phi
c       note the x forcing (f) is centered on the equator, as is u.
c       the phi funtions are calculated in between the f's
c
      parameter(m=78,mp=79,n=119,np=120)
      common/p1/phi(np)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/g1/gsum(n)
      sum=0.
      do 10 i=1,n-1
      ip=i+1
  10  sum=sum+2.*(gsum(i)*phi(i)+gsum(ip)*phi(ip))+phi(i)*
     1  gsum(ip)+phi(ip)*gsum(i)
      sum=sum*dy/6. + dy*(gsum(1)+gsum(n))*(3.*phi(n)+
     1  phi(np))/8.
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine solve
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

       subroutine solve(k)
       parameter(m=78,mp=79,n=119,np=120)
       double precision w(n),d
       common/so/d(n)
       common/m3/xl(3,np),g(np)
       common/m5/u(mp,n),v(m,np),h(mp,n)
c
c   this routine solves the tridiagonal matrix xl*v=g using
c   gausian elimination (see Holton's notes 8 Oct 84). the g
c   matrix is the forcing (the right hand side of 4.7 in C&P),
c   the xl is the one calculated Laplacian on the LHS of 4.7,
c   and v is the meridional velocity (v=0 at j=1,np)
c       note the interval is x=o at i=1 x=l at j+1
c       so g(2)--g(n)  and xl(i,2)--xl(i,n) are used to find
c       v(i,2) to v(i,n); v(i,1)=v(i,np)=0

c   get w(i)
       w(1)=0.
       do 20 i=2,n
  20   w(i)=(g(i)-xl(1,i)*w(i-1))/(xl(2,i)-xl(1,i)*d(i-1))
c   v(k,1)=v(k,np)=0
       do 30 i=n,2,-1
  30   v(k,i)=w(i)-d(i)*v(k,i+1)
       return
       end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine fixros
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine fixros(i)
c
c   I don't know what this routine does.  There's some sort of projection
c   onto the meridional structures of waves (phi's) but that's about all
c   I can figure out.
c
      parameter(m=78,mp=79,n=119,np=120)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/p1/phi(np)
      integer p
      sum=0.
      do 50 j=1,n-1
   50 sum=sum+2.*((hn(i,j)+un(i,j))*phi(j)+phi(j+1)*(hn(i,j+1)
     1  +un(i,j+1)))+ (hn(i,j)+un(i,j))*phi(j+1) + (hn(i,j+1)+
     2  un(i,j+1))*phi(j)
      rfix=sum*dy/12. +dy*((3.*phi(1)+phi(np))*(un(i,1)+hn(i,1))+
     1  (3.*phi(n)+phi(np))*(un(i,n)+hn(i,n)))/16.
      do 20 j=1,n
      un(i,j)=un(i,j)-rfix*phi(j)
  20  hn(i,j)=hn(i,j)-rfix*phi(j)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine westbc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine westbc(aknew)
c
c  now get the western boundary condition for the next time step
c   where un is that of the rossby wave only
c
      parameter(m=78,mp=79,n=119,np=120)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      sum=0.
cccc      do 50 j=1,n
      do 50 j=2,n-1
   50 sum=sum+un(1,j)
ct80  aknew=-sum*dy/cc
coriginal      aknew=-0.8*sum*dy/cc
cB88        aknew = -1*sum*dy/cc
      aknew=-0.8*sum*dy/cc

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine switch
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine switch(aknew)
c
c  in this subrouine we multiply the kelvin and rossby response at
c   t=np  by the frictional factor and switch the
c   arrays from ..p to ..
c
      parameter(m=78,mp=79,n=119,np=120)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/o1/ua(m,n),va(mp,np),wndnon,wndno2,fert2
      common/m2/f(m,n),fert
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/m5/u(mp,n),v(m,np),h(mp,n)
      integer p
      in=alp+1
      akp(1)=aknew
      do 11 i=2,in
  11  akp(i)=akp(i)+ak(1)+(aknew-ak(1))*(1.-(i-1)/alp)
      do 10 i=1,mp
      ak(i)=akp(i)*fert
      do 10 j=1,n
      u(i,j)=un(i,j)*fert
  10  h(i,j)=hn(i,j)*fert
      do 30 i=1,m
      do 30 j=1,np
  30  v(i,j)=v(i,j)*fert2
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine printe
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine printe(ntime,c,hh0)
c
c   I (DV) think this routine computes the complete fields in physical
c   space, after having calculated the wave amplitudes in wave space.
c   It looks like it also calculates the temperature field, etc.  An older
c   version of this file (commented out) plotted some results (plotem).
c
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      parameter(m=78,mp=79,n=119,np=120)
      common/o1/ua(m,n),va(mp,np),wndnon,wndno2,fert2
      common/m2/f(m,n),fert
      common/m4/gwn(mp,np),un(mp,n),hn(mp,n)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common/m1/dx,dy,cc,a1,a2,a3,y(n)
      common/p1/phi(np)
      common/tem1/tn(nx,ny),w1(nx,ny),tza(26),hbar(26)
      common/s10/us(26,30),vs(26,30),hs(26,30),rs,yy(30),cof(n)
      dimension temp(26,np),temp2(mp,2)
      common/pr/utot(26,30),htot(26,30),vtot(26,30),u1(26,30),
     1  h1(26,30),v1(26,30)
      common/neww/u1big(mp,n),v1big(m,np),wbig(m,n),tem(m,30)
      integer p
      character*80 label
      data dxdim,dydim/220000.,55000./
c      call energy(ntime)
      do 210 j=1,n
      u1big(1,j)=c*(u(1,j)+ak(1)*phi(j)+cof(j)*(y(j)*(gwn(1,j)+
     &  gwn(1,j+1))*fert/2.+rs*fert2*f(1,j)))
      u1big(mp,j)=c*(u(mp,j)+ak(mp)*phi(j)+cof(j)*(y(j)*(gwn(mp,j)+
     &  gwn(mp,j+1))*fert/2.+rs*fert2*f(m,j)))
      do 210 i=2,m
  210 u1big(i,j)=c*(u(i,j)+ak(i)*phi(j)+cof(j)*(y(j)*(gwn(i,j)+
     & gwn(i,j+1))*fert/2.+rs*fert2*(f(i,j)+f(i-1,j))/2.))
      do 215 i=1,m
      v1big(i,1)=c*(v(i,1)+(cof(1)-(cof(2)-cof(1))/2.)*(rs*fert*
     & (gwn(i,1)+gwn(i+1,1))/2.-(y(1)-dy/2.)*fert2*f(i,1)))
      v1big(i,np)=c*(v(i,np)+(cof(n)-(cof(n-1)-cof(n))/2.)*(rs*fert*
     & (gwn(i,np)+gwn(i+1,np))/2.-(y(n)+dy/2.)*fert2*f(i,n)))
      do 215 j=2,n
  215 v1big(i,j)=c*(v(i,j)+(cof(j-1)+cof(j))*(rs*fert*(gwn(i,j)+
     & gwn(i+1,j))/2.-(y(j)+y(j-1))*fert2*(f(i,j)+f(i,j-1))/4.)/2.)
      do 11 j=1,n
      do 11 i=1,26
      is=i*3-2
  11  temp(i,j)=(u1big(is,j)+u1big(is+3,j))/6.+(u1big(is+1,j)
     & +u1big(is+2,j))/3.
      do 12 i=1,26
      u1(i,1)=(temp(i,2)+temp(i,4))/4.+temp(i,3)/2.
      u1(i,30)=(temp(i,n-1)+temp(i,n-3))/4.+temp(i,n-2)/2.
      do 12 j=2,29
      jt=j*4
  12  u1(i,j)=((temp(i,jt)+temp(i,jt-4))/2.+temp(i,jt-1)+
     1 temp(i,jt-2)+temp(i,jt-3))/4.

      do 21 j=1,n
      do 21 i=1,26
      is=i*3-2
  21  temp(i,j)=(h(is,j)+h(is+3,j)+(ak(is)+ak(is+3))*phi(j))/6.
     1 +(h(is+1,j)+h(is+2,j)+(ak(is+1)+ak(is+2))*phi(j))/3.
      do 22 i=1,26
      htot(i,1)=hh0*((temp(i,2)+temp(i,4))/4.+temp(i,3)/2.)
      htot(i,30)=hh0*((temp(i,n-1)+temp(i,n-3))/4.+temp(i,n-2)/2.)
      do 22 j=2,29
      jt=j*4
  22  htot(i,j)=hh0*(((temp(i,jt)+temp(i,jt-4))/2.+temp(i,jt-1)+
     1 temp(i,jt-2)+temp(i,jt-3))/4.)
      do 31 j=1,np
      do 31 i=1,26
      is=i*3
  31  temp(i,j)=(v1big(is-2,j)+v1big(is-1,j)+v1big(is,j))/3.
      do 32 i=1,26
      do 32 j=1,30
      js=j*4
  32  v1(i,j)=(temp(i,js-3)+temp(i,js-2)+temp(i,js-1)+temp(i,js))/4.
      do 216  i=1,m
      do 216 j=1,n
 216  wbig(i,j)=50.*((u1big(i+1,j)-u1big(i,j))/dxdim+(v1big(i,j+1)
     &  -v1big(i,j))/dydim)
      do 217 i=1,m
      tem(i,1)=(wbig(i,1)+wbig(i,2)+wbig(i,3)+wbig(i,4)/2.)/3.5
      tem(i,30)=(wbig(i,n-3)/2.+wbig(i,n-2)+wbig(i,n-1)+wbig(i,n))/3.5
      do 217 j=2,29
      jj=(j-1)*4
  217 tem(i,j)=((wbig(i,jj)+wbig(i,jj+4))/2.+wbig(i,jj+1)+wbig(i,jj+2)
     &  +wbig(i,jj+3))/4.
      do 218 i=1,26
      ii=i*3-2
      do 218 j=1,30
 218  w1(i,j)=(tem(ii,j)+tem(ii+1,j)+tem(ii+2,j))/3.
c now write the vertical velocity to file, stored in w1
c      write()ntime,w1
      write(label,950)ntime
      llabel=23
      not=ntime/90
      ntemp=not*90
      if(ntime.eq.ntemp)then
c      write(label,950)ntime
c       call plotem(u,mp,n,label,llabel)
c      write(label,951)ntime
c       call plotem(h,mp,n,label,llabel)
c      call plotem(v,m,np,label,llabel)
       else
       endif
      do 40 i=1,mp
      temp2(i,2)=hh0*(h(i,59)+h(i,60)+h(i,61)+ak(i)*(phi(59)+
     1  phi(60)+phi(61)))/3.
  40  temp2(i,1)=c*(u(i,59)+u(i,60)+u(i,61)+ak(i)*(phi(59)+
     1  phi(60)+phi(61)))/3.
c      if(mod(ntime,30).eq.0)      write(15)ntime,temp2

c      call ezmxy(y,temp2,n,2,n,'the height field at time 1800$')



  900 format(2x,10f8.4)
  901 format(20(2x,15f7.4,/))
  902 format(20(2x,16f7.2,/))
  910 format(' the h field')
  920 format(' the fields at time =',i7)
  930 format(' the u field')
  940 format(' the v field')
  950 format('u field at time=',i7)
  951 format('h field at time=',i7)
  952 format('v field at time=',i7)
  960 format('surface u at t =',i7)
  961 format('surface v at t =',i7)
  953 format('surface w at t =',i7)
  963 format('surface w at t =',i7)
  964 format(i5,13f9.6)
c  965 format(' the u1, v1, and h fields at t = ',i6,' are written'
c     1 ,' to tape')
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine thermo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine thermo(ntime,iseed)
c
c   This program calculates the temperature tendency for the upper level.
c   Note that it also includes a call to the random number generator, to
c   add noise to the stable version of the model.
c
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      parameter(nxp1=27,nxp2=28,nyp2=32,nyp3=33,nyp4=34,amp=0.)
c   amp should correspond to the standard deviation of the SST noise
      real rnum,rnum2(nxp2,nyp4),rnum2b(nxp2,nyp4)
      real nin(8,3),ninr(8,2)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)
      common/tem2/ubar(26,30,12),vbar(26,30,12),wbar(26,30,12),
     1 wave(26,30),tbar(26,30,12)
      common/pr/uave(26,30),h(26,30),vave(26,30),u(26,30),
     1 have(26,30),v(26,30)
      common /coup2/ to(26,30),tave(26,30)
      common/coup3/fsns(26,30,12),uinit(26,30,12),vinit(26,30,12),
     1  fsnsf(26,30)
      common/bigdom/fsnsb(60,30,12),fsnsbf(60,30),tbig(60,30),
     1  tbarb(60,30,12),tbarbf(60,30)
      common /rannum/ rnum3(nxp2,nyp4)
      common/wrk1/ tw1(26,30),tw2(26,30)
      dimension a(2),b(2)
      data a,b,g1,g2,h1/28.,-40.,.0125,.03,.75,.75,50./
c      data dx,dy,tdx,tdy,dt,alpt/6.6e05,2.2e05,1.32e06,4.4e05,
c     1  2.16e05,9.2593e-08/
c Note:  alpt is temperature damping time scale.  The three 
c choices are 90day, 110day, and 125day (all ^-1).  DV 052108
      data dx,dy,tdx,tdy,dt,alpt/6.6e05,2.2e05,1.32e06,4.4e05,
c     1  2.16e05,1.28601e-07/
c     1  2.16e05,9.2593e-08/
c     1  2.16e05,1.05219e-07/
     1  2.16e05,1.28601e-07/
c
c  nin are the nino 1,3,4 indicies (nin(x,index)), where:
c    nin(1,y) is the advecive contribution
c    nin(2,y) is the anomalous upwelling contribution
c    nin(3,y) is the subsurface contribution
c    nin(4,y) is the temperature change after 10 days
c
c  note that it is assumed that u,v,w, and h are the time n+1 dimensional
c  current (layer one) and upper layer height fields
c  ubar,vbar,wbar and hbar are the monthly averaged mean fields read in by
c  readu
      nnn=mod((ntime-6),360)+1

c  Initialize rnum3 - just to be safe
      if(ntime.le.10) then
      do 160 i=1,nxp2
      do 160 j=1,nyp4
 160  rnum3(i,j) = 0.
      endif
c
c  now get the average fields during the time of temperature
c  advection

      call initm(nnn,mon0u,mon1u,mon0v,mon1v)
      call clock(nnn,mon0u,mon1u,uinc)
      call avecur(mon0u,mon1u,uinc)
      call zeroua(nin,8,3)
      call zeroua(ninr,8,2)
c  integerate the temperature equation
      do 100 ijk=1,4
c  calculate advection of total T by anomalous currents
      do 10 j=1,ny
      if(u(1,j).lt.0.)then
      tn(1,j)=to(1,j)-dt*(u(1,j)+u(2,j))*(to(2,j)+tave(2,j)-
     1  (tave(1,j)+to(1,j)))/tdx
      else
      tn(1,j)=to(1,j)
      endif
      if(u(nx,j).ge.0.)then
      tn(nx,j)=to(nx,j)-dt*(u(nx,j)+u(nxn,j))*(to(nx,j)+tave(nx,j)
     1  - (tave(nxn,j)+to(nxn,j)))/tdx
      else
      tn(nx,j)=to(nx,j)
      endif
      do 10 i=2,nxn
      ip=i+1
      im=i-1
      if(u(i,j).ge.0.)then
      tn(i,j)=to(i,j)-dt*(u(im,j)+u(i,j))*(to(i,j)+tave(i,j)-
     1 (tave(im,j)+to(im,j)))/tdx
      else
      tn(i,j)=to(i,j)-dt*(u(i,j)+u(ip,j))*(to(ip,j)+tave(ip,j)-
     1 (tave(i,j)+to(i,j)))/tdx
      endif
  10  continue

      do 20 i=1,nx
      if(v(i,1).lt.0.)tn(i,1)=tn(i,1)-dt*(v(i,2)+v(i,1))*(to(i,2)+
     1 tave(i,2)-(tave(i,1)+to(i,1)))/tdy
      if(v(i,ny).ge.0.)tn(i,ny)=tn(i,ny)-dt*(v(i,ny)+v(i,nyn))*
     1   (to(i,ny)+tave(i,ny)-(tave(i,nyn)+to(i,nyn)))/tdy
      do 20 j=2,nyn
      jm=j-1
      jp=j+1
      if(v(i,j).ge.0.)then
      tn(i,j)=tn(i,j)-dt*(v(i,j)+v(i,jm))*(to(i,j)+tave(i,j)-(
     1  tave(i,jm)+to(i,jm)))/tdy
      else
      tn(i,j)=tn(i,j)-dt*(v(i,j)+v(i,jp))*(to(i,jp)+tave(i,jp)-
     1 (tave(i,j)+to(i,j)))/tdy
      endif
  20  continue

c calculate advection of anomalous T due to mean currents
      do 110 j=1,ny
      if(uave(1,j).lt.0.)tn(1,j)=tn(1,j)-dt*(uave(1,j)
     1  +uave(2,j))*(to(2,j)-to(1,j))/tdx
      if(uave(nx,j).ge.0.)tn(nx,j)=tn(nx,j)-dt*(uave(nx,j)
     1   +uave(nxn,j))*(to(nx,j)-to(nxn,j))/tdx
      do 110 i=2,nxn
      ip=i+1
      im=i-1
      if(uave(i,j).ge.0.)then
      tn(i,j)=tn(i,j)-dt*(uave(im,j)+uave(i,j))*(to(i,j)-to(im,j))/tdx
      else
      tn(i,j)=tn(i,j)-dt*(uave(i,j)+uave(ip,j))*(to(ip,j)-to(i,j))/tdx
      endif
  110 continue

      do 120 i=1,nx
      if(vave(i,1).lt.0.)tn(i,1)=tn(i,1)-dt*(vave(i,2)+vave(i,1))*
     1  (to(i,2)-to(i,1))/tdy
      if(vave(i,ny).ge.0.)tn(i,ny)=tn(i,ny)-dt*(vave(i,ny)+
     1  vave(i,nyn))*(to(i,ny)-to(i,nyn))/tdy
      do 120 j=2,nyn
      jm=j-1
      jp=j+1
      if(vave(i,j).ge.0.)then
      tn(i,j)=tn(i,j)-dt*(vave(i,j)+vave(i,jm))*(to(i,j)-to(i,jm))/tdy
      else
      tn(i,j)=tn(i,j)-dt*(vave(i,j)+vave(i,jp))*(to(i,jp)-to(i,j))/tdy
      endif
 120  continue
c      call nin1(sum1,sum2,sum3)
c      nin(1,1)=sum1+nin(1,1)
c      nin(1,2)=sum2+nin(1,2)
c      nin(1,3)=sum3+nin(1,3)

      do 30 i=1,nx
      do 30 j=1,ny
      nflag=nf0(h(i,j))
cho      tsub=a(nflag)*(tanh(b(nflag)*(hbar(i)+abs(h(i,j))))-tanh(b(nflag)*
cho     1  hbar(i)))
      tsub=a(nflag)*(tanh(b(nflag)*(hbar(i)+1.5*abs(h(i,j))))-
     1  tanh(b(nflag)*hbar(i)))
      wtot=wave(i,j)+w(i,j)
      tw1(i,j)=dt*g1*(f1(wtot)-f1(wave(i,j)))*tza(i)
      tw2(i,j)=dt*g2*f1(wtot)*(to(i,j)-tsub)/h1
c  30  tn(i,j)=tn(i,j)-dt*alpt*to(i,j)-tw1(i,j)-tw2(i,j)+
c     1  dt*fsnsf(i,j)/(1026.*3850.*h1) 
cfsns  30  tn(i,j)=tn(i,j)-dt*alpt*to(i,j)-tw1(i,j)-tw2(i,j)+
cfsns     1  vinit(i,j,1)/(1026.*3850.*h1) 
  30  tn(i,j)=tn(i,j)-dt*alpt*to(i,j)-tw1(i,j)-tw2(i,j)
c ADD Big Domain Forcing
      do 36 i=1,60
      do 36 j=1,30
 36   tbig(i,j)=fsnsbf(i,j)
c      tsub=tbig(i,j)-dt*alpt*tbig(i,j)+
c     1 dt*fsnsbf(i,j)/(1026.*3850.*h1)
c 36   tbig(i,j)=tsub

c  Swap tn with to
      do 33 i=1,nx
      do 33 j=1,ny
 33   to(i,j)=tn(i,j)

c  Continue the 2.5 day timestep calculation of temperature (ijk)
 100  continue

c  DV 070907
c  Random number generator
      do 51 i=1,nxp2 
      do 51 j=1,nyp4
c  Normally distributed random variable
      rnum2(i,j) = 0.
      rnum2b(i,j) = 0.
      do 48 k = 1,100
      call random_number(rnum)
      rnum2(i,j) = rnum2(i,j)+rnum
 48   continue
      rnum2(i,j) = ((rnum2(i,j)/100.)-0.5)*(sqrt(100.)/0.2887)
 51   continue

c  Apply spatial smoother:  1-2-1 filter, once in merid, once in zonal, 
c  and two more times in the meridional direction.
      do 55 kit=1,3
      do 52 i=1,nxp2 
      do 52 j=2,nyp3
 52   rnum2b(i,j) = 0.25*rnum2(i,j-1)+0.5*rnum2(i,j)+0.25*rnum2(i,j+1)
      do 59 i=1,nxp2
      do 59 j=1,nyp4
 59   rnum2(i,j) = rnum2b(i,j)
      do 53 i=2,nxp1
      do 53 j=1,nyp4
 53   rnum2b(i,j) = 0.25*rnum2(i-1,j)+0.5*rnum2(i,j)+0.25*rnum2(i+1,j)
      do 54 i=1,nxp2
      do 54 j=1,nyp4
 54   rnum2(i,j) = rnum2b(i,j)
c      do 54 j=3,nyp2
c 54   rnum2(i,j) = 0.25*rnum2(i,j-1)+0.5*rnum2(i,j)+0.25*rnum2(i,j+1)
c      do 55 j=3,nyp2
c 55   rnum2(i,j) = 0.25*rnum2(i,j-1)+0.5*rnum2(i,j)+0.25*rnum2(i,j+1)
c  Add temporal persistence to rnum3
 55   continue
      do 159 i=1,nxp2
      do 159 j=1,nyp4
c      write(*,*) rnum3(i,j)
 159  rnum3(i,j) = 0.6*rnum3(i,j)+0.8*rnum2(i,j)
c  Add random noise to ocean temp, and swap into 'to'.
      do 56 i=1,nx
      do 56 j=1,ny
 56   to(i,j)=to(i,j)+amp*rnum3(i+1,j+2)
c      call nin2

cddd
c      if(mod(ntime,30).eq.0)      write(16)ntime,nin
c      if(mod(ntime,30).eq.0)      write(19,*)ntime,to(24,15)
      if(mod(ntime,30).eq.0) then
      do 57 i=13,20
      do 57 j=13,18
         tem = tem+to(i,j)
 57   continue
      tem = tem/48.
      write(19,*)ntime,tem
      if(.true.) then
      do 58 j=1,30
      write(27,974)(to(i,j),i=1,26)
 58   write(28,974)(h(i,j),i=1,26)
      endif
      endif

 974  format(30e12.4)

      return
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  functions nf0 and f1
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 
       function nf0(h)
       if(h.ge.0.)then
       nf0=1
       else
       nf0=2
       endif
       return
       end

       function f1(w)
       if(w.gt.0.)then
       f1=w
       else
       f1=0.
       endif
       return
       end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine avecur
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine avecur(mon0,mon1,xinc)
c
c   Interpolates the currents to mid-monthly values
c
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common/tem2/ubar(26,30,12),vbar(26,30,12),wbar(26,30,12),
     1 wave(26,30),tbar(26,30,12)
      common/pr/uave(26,30),h(26,30),vave(26,30),u(26,30),
     1 have(26,30),v(26,30)

      common /coup2/ to(26,30),tave(26,30)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)
c
c  ubar, vbar etc are the monthly mean current fields
c
c      write(11,901)mon0,mon1,xinc
c 901  format('fields interpolated between mon=',i2,' and ',
c     1 i2,' xinc=',e12.4)
      do 10 i=1,26
      do 10 j=1,30

c uave,tave,vave,wave are the average ocean fields at the time of interest
      uave(i,j)=ubar(i,j,mon0)+xinc*(ubar(i,j,mon1)-ubar(i,j,mon0))
      tave(i,j)=tbar(i,j,mon0)+xinc*(tbar(i,j,mon1)-tbar(i,j,mon0))
      vave(i,j)=vbar(i,j,mon0)+xinc*(vbar(i,j,mon1)-vbar(i,j,mon0))
  10  wave(i,j)=wbar(i,j,mon0)+xinc*(wbar(i,j,mon1)-wbar(i,j,mon0))
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine nin2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine nin2
c
c  Calculate appropriate nino indices.  Uncomment the write statement to
c  output the index.
c
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)

c nino one index
      j=26
      sum1=(tn(j,13)/2.+3*tn(j,12)/2.+tn(j,11))/3.
c nino three index
      sum2=0.
      do 10 i=16,25
  10  sum2=sum2+(tn(i,13)+tn(i,18))/2.+tn(i,14)+tn(i,15)+tn(i,16)
     1  +tn(i,17)
      sum2=sum2/50.
c nino four index
      sum3=0.
      do 20 i=11,15
  20  sum3=sum3+(tn(i,13)+tn(i,18))/2.+tn(i,14)+tn(i,15)+tn(i,16)
     1  +tn(i,17)
      sum3=sum3/25.

c      write(56,*)sum2
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine atmos
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c
c  DV note:  In general, to convert between stable, neutral, and unstable
c  versions of the model, search for cstable, cneut, or cunstable.  Copy
c  the appropriate line, and uncomment it.
c
c  Now have changed Cd and reflection coefficient and mechanical damping
c  to make this the T80 model (sort of). See comments ct80
c
c has diveru changes noted by tony
c****************************************11/21 modified for DB's data
c Has David's updated method of calculating thermodynamic
c contributions; these are written to therm.big ************
c this is the same as atm.coup.f but without the plotting subroutines
c moved from the prime on February 7
ccc the new nondimensionalization is in place
cc note the atmospheric initial heating field is now given by the
cc linear Clausius Claperon Equation.
c the thermal coupling is now alp=1.4, qbeta =.75
cddd a new alp is used in the intitial heating
c new qfilter acts every time loop=0
c  new q filter and now heating shuts off on loop=0 if divmax<divcr
        subroutine atmos(ntime)
c    ATMOSPHERIC LINEAR DAMPED MODEL FOR A GIVEN SST
c   this program caculates the anomalous wind response ua,va due
c   to an anomalous SST with the model a la Zebiak (1986). Hence the
c   mean sst is also needed as well as the mean monthly wind fields
c Note, that since we are limiting
c the heating/forcing of the atmosphere to occer over the oceanis Pacific
c basin, the only contribution to ua,va we need to store is that at grids
c (x=1,26;y=21,50)
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex a,cz,qf,vf,uf,g,z,c1
      common/ag/g(np),vf(np),uf(np),qf(np)
      common/ah/dyn,ep,c1,aaa,xl,e2
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/ad/u(mp,np),v(mp,np)
      common/coup/ua(26,30),va(26,30)

      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
c the sst anomalies are read in on tape 23
c the wind anomalies on the Pacific Ocean large grid are
c output on file (26), and the darwin and Tahiti pressure
c anomalies (not currently working) are output on (27)
c the annual wind and sst fields (monthly averaged) are
c read on files (24) and (25)

c  q is the nondimensional heating on a 6 deg by 2 deg grid
c  the grid is thus 360/6+1=61 in x   by   140/2=70 in y
c  the y grid point n=70 is at 1 degree south
c  and m=1 is at 127 E. The foring due to q is only
c  from m=1,26 to n=56,85 all other values of q should
c  always be Zero.
C
c kp-1 is the number of fourier modes to use and
c mpm=mp-1

c  read in the correct mean and anomaly fields needed for the calculation
      len=5.*1024/60.
      new=mod((ntime-10),30)
      call settem(new)
c  calculate the anomalous heating field q0 from SST
c      call getq(qdar,qtah,new)
      call getq2(qdar,qtah,new)
cddd      call qfilter
c      call plotq(q,mp,np,.05,.45,1,1,ntime)
c this is where we loop to get the convergene
coriginal      do 300 loop=1,3
      do 300 loop=1,3
c      write(11,904)loop
      call zeroua(u,mp,np)
      call zeroua(v,mp,np)
c now for each latitude we do a fft, then solve the meridional
c structure eqn for each zonal wave number k
cddd
      call qfilter

      do 10 k=1,kp
      xkn=(k-1)*xl/aaa
      call fft(k)
c      write(11,920)k
c  now get the meridional structure of wavenumber k
      call solvev(k)
c add this to the wind anomaly uv for iteration number loop
  10  call adduv(k)
c add the wind anomaly for that iteration to the total anomaly
c
c get the divergence from the last loop
      call diver
c check for convergence
      call divck(nflag,loop)
c if nflag=0, no convergence
c if nflag=1,    convergence if nlag=2, instability developed
      if(nflag.ge.1)go to 301
      call iter(qdar,qtah)
c      if((loop.eq.1).and.(new.ne.0))
c     1    call plotv(u,v,mp,np,len,loop,mon,ntime)
c      if((loop.eq.1).and.(new.ne.0))
c     1   call plotq(q,mp,np,.05,.45,0,1,loop)
c      if((loop.eq.1).and.(new.ne.0))
c     1  call plotq(div,26,30,.55,.95,1,2,loop)
 300  continue
 301  continue
c check to see if iteration is making divergence field unrealistically
c  big (ie, the calculation is going unstable)
      if(nflag.eq.2)write(11,954)
  954 format(' The divergence is larger than 7e-06')
      loop =loop-1
c      call plotv(ua,va,26,30,len,loop,mon,ntime)
c      call plotq(q,mp,np,.05,.45,0,1,loop)
      if(new.ne.0)then
      call adduva
      else

      do 200 j=1,30
      jp=j+20
      do 200 i=1,26
      ua(i,j)=u(i,jp)
  200  va(i,j)=v(i,jp)
      endif
      call diveru
c      call plotq(divnm,26,30,.55,.95,1,2,loop)
cddd      write(26)ntime,ua,va
c now calculate the anomalous pressure at tahiti and darwin
c note something is presently wrong here
      itah=15
      jtah=7
      ptah=-1.25*60.*(divv(ua(itah+1,jtah),ua(itah-1,jtah),va(itah,
     1 jtah+1),va(itah,jtah-1))*xl+qtah)/ep
      idar=2
      jdar=9
      pdar=-1.25*60.*(divv(ua(idar+1,jdar),ua(idar-1,jdar),va(idar,
     1 jdar+1),va(idar,jdar-1))*xl+qdar)/ep
c      if(mod(ntime,30).eq.0)      write(27)ntime,ptah,pdar
c      write(11,983)ntime,ptah,pdar
c fix q here
c      write(11,940)
c      do 800 j=1,np,2
c  800 write(11,930)j,(u(i,j),i=1,mp,2)
c      write(11,941)
c      do 810 j=1,np,4
c  810 write(11,930)j,(v(i,j),i=1,mp,2)
 930  format(' j=',i4,/,10(5e12.4,/))
 940  format(' the u field')
 941  format(' the v field')
 900  format(i1,i2)
  904 format(' loop=',i3)
 920  format(' ok to here  k=',i2)
 983  format(' month = ',2x,i2,' p(tah) =',e12.4,'  p(dar)=',e12.4)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine settem
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine settem(new)
c
c  Calculate the mean divergence from the mean winds.  This is used in the
c  atmospheric heating iteration.
c
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common/wnd1/um(26,30),vm(26,30)
      do 40 j=2,29
      divmn(1,j)=divv(um(2,j),um(1,j),0.,0.)*2.+
     1  divv(0.,0.,vm(1,j+1),vm(1,j-1))
      divmn(26,j)=divv(um(26,j),um(25,j),0.,0.)*2.+
     1  divv(0.,0.,vm(26,j+1),vm(26,j-1))
        do 40 i=2,25
  40  divmn(i,j)=divv(um(i+1,j),um(i-1,j),vm(i,j+1),vm(i,j-1))
      do 50 i=2,25
      divmn(i,1)=divv(um(i+1,1),um(i-1,1),0.,0.)+
     1  divv(0.,0.,vm(i,2),vm(i,1))*2.
  50  divmn(i,30)=divv(um(i+1,30),um(i-1,30),0.,0.)+
     1  divv(0.,0.,vm(i,30),vm(i,29))*2.
      divmn(1,1)=divv(um(2,1),um(1,1),0.,0.)*2.+
     1  divv(0.,0.,vm(1,2),vm(1,1))*2.
      divmn(1,30)=divv(um(2,30),um(1,30),0.,0.)*2.+
     1  divv(0.,0.,vm(1,30),vm(1,29))*2.
      divmn(26,1)=divv(um(26,1),um(25,1),0.,0.)*2.+
     1  divv(0.,0.,vm(26,2),vm(26,1))*2.
      divmn(26,30)=divv(um(26,30),um(25,30),0.,0.)*2.+
     1  divv(0.,0.,vm(26,30),vm(26,29))*2.
c      write(11,906)
c  906 format('now plotting mean divergence')
c      call plotq(divmn,26,30,.55,.95,1,2,0)
      if(new.eq.0)return
      do 60 i=1,26
      do 60 j=1,30
  60  divmn(i,j)=divmn(i,j)+divnm(i,j)

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine getq
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine getq(qdar,qtah,new)
c this subroutine calculates the intital heating anomaly qo due
c to the sst=ta+tbar, where ta is the anomaly and tbar is the mean field
c at this time
c
c             t(ref)**2
c  q= alp*ta* ---------  * exp(-b(1/t(ref)-1/t(mean))
c             t(mean)**2
c
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex z,cz
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common /coup2/ ta(26,30),tbar(26,30)
cddd
corig      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,1.6,32.58/
cuncoup      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,0.0,32.58/
      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,1.4,32.58/
c      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,1.4,32.58/
      do 10 i=1,26
      do 10 j=21,50
      qo(i,j)=alp*ta(i,j-20)*exp(b/tref-b/(tbar(i,j-20)+tkel))*
     1  (tref/(tbar(i,j-20)+tkel))**2
c      qo(i,j)=alpnl*exp(b/tref-b/(tbar(i,j-20)+tkel))*
c     1 (exp(b*ta(i,j-20)/(tbar(i,j-20)+tkel)**2)-1)
  10  q(i,j)=qo(i,j)
      qdar=qo(2,29)
      qtah=qo(15,27)
      if(new.eq.0)then
      do 30 i=1,26
      do 30 j=21,50

celena  30  qonm(i,j)=qo(i,j)
   30  qonm(i,j-20)=qo(i,j)

      else
      do 20 i=1,26
      do 20 j=21,50
celena      q(i,j)=q(i,j)-qonm(i,j)
      q(i,j)=q(i,j)-qonm(i,j-20)
celena      qonm(i,j)=qo(i,j)
      qonm(i,j-20)=qo(i,j)
  20  qo(i,j)=q(i,j)
      endif
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine getq2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine getq2(qdar,qtah,new)
c this subroutine calculates the intital heating anomaly qo due
c to the sst=ta+tbar, where ta is the anomaly and tbar is the mean field
c at this time
c
c             t(ref)**2
c  q= alp*ta* ---------  * exp(-b(1/t(ref)-1/t(mean))
c             t(mean)**2
c
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex z,cz
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common /coup2/ ta(26,30),tbar(26,30)
      common/bigdom/fsnsb(60,30,12),fsnsbf(60,30),tbig(60,30),
     1  tbarb(60,30,12),tbarbf(60,30)
cddd
corig      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,1.6,32.58/
cuncoup      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,0.0,32.58/
      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,1.4,32.58/
c      data tref,tkel,b,alp,alpnl/303.,273.,5.4e+03,1.4,32.58/
      do 10 i=1,60
      do 10 j=21,50
      qo(i,j)=alp*tbig(i,j-20)*exp(b/tref-b/(tbarbf(i,j-20)+tkel))*
     1  (tref/(tbarbf(i,j-20)+tkel))**2
c      qo(i,j)=alpnl*exp(b/tref-b/(tbar(i,j-20)+tkel))*
c     1 (exp(b*ta(i,j-20)/(tbar(i,j-20)+tkel)**2)-1)
  10  q(i,j)=qo(i,j)
      do 11 j=21,50
      qo(61,j)=qo(1,j)
  11  q(61,j)=qo(1,j)
      do 12 i=1,26
      do 12 j=21,50
      qo(i,j)=alp*(tbig(i,j-20)+ta(i,j-20))*
     1  exp(b/tref-b/(tbarbf(i,j-20)+tkel))*
     1  (tref/(tbarbf(i,j-20)+tkel))**2
c      qo(i,j)=alpnl*exp(b/tref-b/(tbar(i,j-20)+tkel))*
c     1 (exp(b*ta(i,j-20)/(tbar(i,j-20)+tkel)**2)-1)
  12  q(i,j)=qo(i,j)

      qdar=qo(2,29)
      qtah=qo(15,27)
      if(new.eq.0)then
      do 30 i=1,26
      do 30 j=21,50

celena  30  qonm(i,j)=qo(i,j)
   30  qonm(i,j-20)=qo(i,j)

      else
      do 20 i=1,26
      do 20 j=21,50
celena      q(i,j)=q(i,j)-qonm(i,j)
      q(i,j)=q(i,j)-qonm(i,j-20)
celena      qonm(i,j)=qo(i,j)
      qonm(i,j-20)=qo(i,j)
  20  qo(i,j)=q(i,j)
      endif
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine qfilter
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine qfilter
c
c  I (DV) think this smooths the heating field - maybe so everything
c  acts nice?
c
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex z,cz
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      dimension work(70), tfil(mp,np)
c
c  first apply two passes of a 1-2-1 filter to the thermal
c  output to in the y direction. Since we don't want to
c  filter the results, but just smooth the output we put
c  the smoothed array in 'tfil'
c
c first y filter
      do 10 i=1,mp
      work(1)=(2*q(i,1)+q(i,2))/3.
      work(np)=(2*q(i,np)+q(i,np-1))/3.
      do 20 j=2,np-1
  20  work(j)=(q(i,j-1)+2*q(i,j)+q(i,j+1))/4.
c second y filter
      tfil(i,1)=(2*work(1)+work(2))/3.
      tfil(i,np)=(2*work(np)+work(np-1))/3.
      do 30 j=2,np-1
  30  tfil(i,j)=(work(j-1)+2*work(j)+work(j+1))/4.
  10  continue
cc first y 1-2-3-2-1 filter
c      do 10 i=1,mp
c      work(1)=(3*q(i,1)+2*q(i,2)+q(i,3))/6.
c      work(2)=(3*q(i,2)+2*(q(i,3)+q(i,1))+q(i,4))/8.
c      work(np)=(3*q(i,np)+2*q(i,np-1)+q(i,np-2))/6.
c      work(np-1)=(3*q(i,np-1)+2*(q(i,np-2)+q(i,np))+q(i,np-3))/8.
c      do 20 j=3,np-2
c  20  work(j)=(3*q(i,j)+2*(q(i,j-1)+q(i,j+1))+q(i,j+2)+q(i,j-2))/9.
c  first 1-2-1 filter in y
c        work(1)=(2*q(i,1)+q(i,2))/3.
c        work(np)=(2*q(i,np)+q(i,np-1))/3.
c        do 20 j=2,np-1
c  20    work(j)=(2*q(i,j)+q(i,j-1)+q(i,j+1))/4.
cc second y 1-2-3-2-1 filter
c      tfil(i,1)=(3*work(1)+2*work(2)+work(3))/6.
c      tfil(i,2)=(3*work(2)+2*(work(3)+work(1))+work(4))/8.
c      tfil(i,np)=(3*work(np)+2*work(np-1)+work(np-2))/6.
c      tfil(i,np-1)=(3*work(np-1)+2*(work(np-2)+work(np))+work(np-3))/8.
c      do 30 j=3,np-2
c  30  tfil(i,j)=(3*work(j)+2*(work(j-1)+work(j+1))+work(j+2)+
c     1 work(j-2))/9.
c      do 31 j=1,np
c  31  tfil(i,j)=work(j)
c  10  continue
c
c now apply a 1-2-1 filter to to in the x direction
c
      do 40 j=1,np
      q(1,j)=(tfil(mp-1,j)+2*tfil(1,j)+tfil(2,j))/4.
      q(mp,j)=q(1,j)
      do 50 i=2,mp-1
  50  q(i,j)=(tfil(i-1,j)+2*tfil(i,j)+tfil(i+1,j))/4.
  40  continue
c      write(xx)ntime,tfil
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine fft
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine fft(k)
c this subroutine calculates the fft of an funcion q(61,y), where
c the point x=61 is also x=1 and is at -pi
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex z,qf,czero,cz,vf,uf,g
      common /aa/czero
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/ag/g(np),vf(np),uf(np),qf(np)
      do 10 i=1,np
  10  qf(i)=czero
      do 20 i=1,np
      do 20 j=1,60
  20  qf(i)=q(j,i)*z(j,k)+qf(i)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine solvev and functions gpro and ucal
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine solvev(k)
c this subroutine sets the forcing array g for the meridional
c structure equation at wave number k. this is used in subroutine
c elim to solve the tridiagonal system.
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex czero,g,uf,vf,qf,gpro,ucal,c1
      common/ag/g(np),vf(np),uf(np),qf(np)
      common /aa/czero
      common/ai/y(np)
      common/ah/dyn,ep,c1,aaa,xl,e2
      xkn=(k-1)*xl/aaa
      g(2)=gpro(czero,qf(2),qf(3),k,y(2))
      g(np-1)=gpro(qf(np-2),qf(np-1),czero,k,y(np-1))
      do 10 i=3,np-2
  10  g(i)=gpro(qf(i-1),qf(i),qf(i+1),k,y(i))
      call elim(k)
c now get the u field at wave number k from the vf field
      uf(1)=czero
      uf(np)=czero
      uf(2)=ucal(czero,vf(2),vf(3),qf(2),xkn,2)
      uf(np-1)=ucal(vf(np-2),vf(np-1),czero,qf(np-1),xkn,np-1)
      do 20 i=3,np-2
  20  uf(i)=ucal(vf(i-1),vf(i),vf(i+1),qf(i),xkn,i)
      return
      end

      complex function gpro(a,b,c,k,y)
c used in elim
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex a,b,c,c1,g1, d,chi
c      complex*16 d,chi
      common /e1/chi(np,kp),d(np,kp),g1(kp)
      common/ah/dyn,ep,c1,aaa,xl,e2
      gpro=dyn*(a-c)/2.+g1(k)*b*y
      return
      end

      complex function ucal(a,b,c,q,xkn,i)
c used in elim
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex a,b,c,q,c1
      common/ai/y(np)
      common/ah/dyn,ep,c1,aaa,xl,e2
      ucal=(ep*y(i)*b/2.+c1*xkn*((c-a)/(2.*dyn)+q))/
     1  (e2+xkn*xkn)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine elim
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine elim(k)
c this subroutine solves a tridiagonal complex matrix for vf
c the points y=1,y=70 have the bc vf=0. and the forcing g(y=1,70)
c also has the property  g(1)=g(70)=0.
c The chi array and an array which will be multipled to the right hand
c side (the forcing q) are set here in atset.
c The forcing array g  is set in subroutine solvev. For an explanation see
c in Holton's notes on numerical methods
c the system  chi(3,n)*v(n+1,n,n-1)=g(n) is solved, where chi(1,n)=chi(3,n)=1
c Chi is a two dimensional matrix and v and g are vectors. n is the meridional
c position and g is the forcing.
      parameter(np=70,kp=20,mp=61,mpm=60,n=69)
c  n=np-1
      complex czero,g,uf,vf,qf,g1, d,chi,czerod,w
      common /aa/czero
      common /aad/czerod
      common /e1/chi(np,kp),d(np,kp),g1(kp)
      common/ag/g(np),vf(np),uf(np),qf(np)
      dimension w(np)
      vf(np)=czero
      vf(1)=czero
      w(1)=czerod
      do 10 j=2,n
 10   w(j)=(g(j)-w(j-1))/(chi(j,k)-d(j-1,k))
      do 20 j=n,2,-1
  20  vf(j)=w(j)-d(j,k)*vf(j+1)
      if(k.eq.1)then
      else
      endif
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine adduv
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine adduv(k)
c this subroutine adds the contribution of the k'th fourier mode
c to the velocity field caluclated during one loop. u and v are
c then the velocitys during this iteration (loop). they must be added
c to the total velocity field ua,va at the end of this itteration
c Note, that since we are limiting
c the heating/forcing of the atmosphere to occer over the oceanis Pacific
c basin, the only contribution to ua,va we need to store is that at grids
c (x=1,27 and x=60;y=21,50)
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex uf,vf,cz,z,g,qf
      common/ag/g(np),vf(np),uf(np),qf(np)
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/ad/u(mp,np),v(mp,np)
      common/coup/ua(26,30),va(26,30)
      do 10 j=19,51
      u(60,j)=u(60,j)+real(uf(j)*cz(60,k))
      v(60,j)=v(60,j)+real(vf(j)*cz(60,k))
      do 10 i=1,27
      u(i,j)=u(i,j)+real(uf(j)*cz(i,k))
  10  v(i,j)=v(i,j)+real(vf(j)*cz(i,k))
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine diver
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine diver
c this subroutine calculates the divergence of the veloctiy
c array only over the Pacific Ocean basin (since it is only
c this which is need to calculate the forcing at the next iteration
c time step. Note that a u,v is needed at position x= 27 and 60
      parameter(np=70,kp=20,mp=61,mpm=60)
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common/ad/u(mp,np),v(mp,np)
      do 10 j=21,50
      div(1,j-20)=divv(u(2,j),u(60,j),v(1,j+1),v(1,j-1))
      do 10 i=2,26
  10  div(i,j-20)=divv(u(i+1,j),u(i-1,j),v(i,j+1),v(i,j-1))
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine divck
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine divck(nflag,n)
c this subroutine checks to see if the diference in the divergence
c fields in the last two iterations is less than some threshold value
c (delcr). if it is then nflag=1, if not nflag=0. If the divergence field
c is greater than some unrealistic value (divcon), then nflag =2.
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common/addd/ divold(26,30)
      data delcr/.15e-06/
      divmax=0.
      nflag=0
      if(n.eq.1)then
      do 5 i=1,26
      do 5 j=1,30
      top=abs(div(i,j))
  5   if(top.gt.divmax)divmax=top
c      write(11,900)n,divmax
      if(divmax.lt.delcr)nflag=1
      go to 100
      else
      do 10 i=1,26
      do 10 j=1,30
      dif=abs(div(i,j)-divold(i,j))
  10  if(dif.gt.divmax)divmax=dif
c      write(11,900)n,divmax
      endif
      if(divmax.gt.delcr)then
      nflag=0
            if(divmax.ge.7.e-06)    nflag=2
      else
      nflag=1
      return
      endif
 100  do 110 i=1,26
      do 110 j=1,30
 110  divold(i,j)=div(i,j)
c      call plotq(divold,26,30,.05,.45,1,2)
 900  format('divmax on loop ', i2,' is ',e12.4)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine iter and function divv
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c  DV Note:  recall that the heating is iterative - once the heating 
c  generates a circulation, the new circulation can influence the mean 
c  divergence, and hence the heating.  This is iterated upon for a few
c  iterations.  This is the 'do 300 loop=1,3' loop.

      subroutine iter(qdar,qtah)
c this subroutine calculates the forcing of the atmosphere in one
c of the iteration loops. q is the forcing for this iteration
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex z,cz
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      do 20 j=1,30
      jp=j+20
      do 20 i=1,26
      divtot=div(i,j)+divmn(i,j)
      if(divmn(i,j).gt.0.)then
           if(divtot.le.0.) then
           q(i,jp)=qo(i,jp)-qbeta*divtot/divcon
           else
           q(i,jp)=qo(i,jp)
           endif
      else
           if(divtot.gt.0.)then
           q(i,jp)=qo(i,jp)+qbeta*divmn(i,j)/divcon
           else
           q(i,jp)=qo(i,jp)-qbeta*div(i,j)/divcon
           endif
      endif
  20  continue
      qtah=qtah+q(15,27)-qo(15,27)
      qdar=qdar+q(2,29)-qo(2,29)
      return
      end

      function divv(u2,u1,v2,v1)
c used in iter
      common/adim/dx,dy
      divv=(u2-u1)/(2.*dx)+(v2-v1)/(2.*dy)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine adduva
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

       subroutine adduva
c this subroutine adds the velocity field calculated during one loop
c (uv) to the total velocity field (ua,va).
c Note, that since we are limiting
c the heating/forcing of the atmosphere to occer over the oceanis Pacific
c basin, the only contribution to ua,va we need to store is that at grids
c (x=1,26;y=21,50)
      parameter(np=70,kp=20,mp=61,mpm=60)
      complex uf,vf,cz,z,g,qf
      common/ag/g(np),vf(np),uf(np),qf(np)
      common /af/ z(mp,kp),q(mp,np),cz(mp,kp),qo(mp,np)
      common/ad/u(mp,np),v(mp,np)
      common/coup/ua(26,30),va(26,30)
      do 10 i=1,26
      do 10 j=1,30
      ua(i,j)=ua(i,j)+u(i,j+20)
  10  va(i,j)=va(i,j)+v(i,j+20)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine diveru
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine diveru
c this subroutine calculates the divergence fo the total ua,va atmospheric
c anomaly field after the iteration process is complete.
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common/coup/ua(26,30),va (26,30)
      do 10 j=2,29
      divnm(1,j)=divv(ua(2,j),ua(1,j),0.,0.)*2+divv(0.,0.,
     1 va(1,j+1),va(1,j-1))
      divnm(26,j)=divv(ua(26,j),ua(25,j),0.,0.)*2+divv(0.,0.,
     1 va(26,j+1),va(26,j-1))
      do 10 i=2,25
  10  divnm(i,j)=divv(ua(i+1,j),ua(i-1,j),va(i,j+1),va(i,j-1))
cnew .. the *2 is new in both of these lines to follow
      do 20 i=2,25
      divnm(i,1)=divv(ua(i+1,1),ua(i-1,1),0.,0.)+divv(0.,0.,
     1 va(i,2),va(i,1))*2
  20  divnm(i,30)=divv(ua(i+1,30),ua(i-1,30),0.,0.)+divv(0.,0.,
     1 va(i,30),va(i,29))*2
cnew
      divnm(1,1)=(divv(ua(2,1),ua(1,1),va(1,2),va(1,1)))*2.
      divnm(26,1)=(divv(ua(26,1),ua(25,1),va(26,2),va(26,1)))*2.
      divnm(1,30)=(divv(ua(2,30),ua(1,30),va(1,30),va(1,29)))*2.
      divnm(26,30)=(divv(ua(26,30),ua(25,30),
     1  va(26,30),va(26,29)))*2.
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine wrstart2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine wrstart2(ntime)
c
c   Write restart file
c
      parameter(m=78,mp=79,n=119,np=120)
      parameter(nnp=70,kp=20,mmp=61,mpm=60)
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common /coup2/ to(26,30),tave(26,30)
      common/coup/uanom(26,30),vanom(26,30)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /af/ z(mmp,kp),q(mmp,nnp),cz(mmp,kp),qo(mmp,nnp)
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      common /rannum/ rnum3(28,34)
      complex a,cz,qf,vf,uf,g,z,c1
      integer p,iseed(2)
      character*10 fmt1
      character*20 rstfnm
c
c   Determine output file name
      write(fmt1,fmt='(a,i1,a)') '(a,i',
     &     int(log10(real((ntime-300)/360+1)+0.5))+1,')'
      write(rstfnm,fmt1) 'Restart/restart.',((ntime-300)/360)+1
c
c   Get most recent iseed
c      call random_seed(get = iseed(1:2))
c
c   Open restart file and write it.
      open(unit=20,file=rstfnm,status='unknown',form='unformatted')
      write(11,900)ntime,20,rstfnm
      rewind(20)
      write(20)ntime,iseed,ak,u,v,h,to,uanom,vanom,
     &     divmn,divnm,qonm,q,div,rnum3
      close(20)

 900  format(' at time ',i8,' a restart file is written',
     1 ' on tape ',i2, ' named ', a)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine wrstart
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine wrstart(ntime,n2rst)
c
c   Write restart file
c
      parameter(m=78,mp=79,n=119,np=120)
      parameter(nnp=70,kp=20,mmp=61,mpm=60)
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common /coup2/ to(26,30),tave(26,30)
      common/coup/uanom(26,30),vanom(26,30)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      common/m5/u(mp,n),v(m,np),h(mp,n)
      common/adiv/div(26,30),divmn(26,30),divcon,qbeta
      common /af/ z(mmp,kp),q(mmp,nnp),cz(mmp,kp),qo(mmp,nnp)
      common/qiter/qonm(26,30),uanm(26,30),vanm(26,30),divnm(26,30)
      complex a,cz,qf,vf,uf,g,z,c1
      integer p
      if(mod(ntime,n2rst).eq.0)then
      ntape=20
      else
      ntape=21
      endif
      write(11,900)ntime,ntape
      rewind (ntape)
      write(ntape)ntime,ak,u,v,h,to,uanom,vanom,divmn,divnm,qonm,q,div
 900  format(' at time ',i6,' a restart file is written',
     1 ' on tape ',i2)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine writeyb
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine writeyb(ijk,nadj)
c
c   Writes output at end
c
      parameter(m=78,mp=79,n=119,np=120)
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      parameter(nmodes=6)    
      common /coup2/ to(26,30),tave(26,30)
      common/qros1/qros(mp,nmodes),qrosp(mp,nmodes)
      common/c1/akp(mp),ak(mp),gamma,alp,p
      dimension totemp(312), aktwo(mp)
      integer p
c
c  write data
c
      indout=indout+1
      print *,'in write mode, ntime, indout =',ijk, indout

c      write(89) to,ak,qros
      ind=0
      do 10 j=10,21
      do 10 i=1,nx
      ind=ind+1
10    totemp(ind)=to(i,j) 
          write(*,*) 'ind = ', ind

      write(81,19) totemp
19    format(312e12.4)

c      do 20 i=1,mp
c20    aktwo(i)=ak(i)*2.
c      write(81,19) totemp,
c     1             (aktwo(ii), ii=1,mp,3),  (qros(ii,2), ii=mp,1,-3),
c     1             (qros(ii,4), ii=mp,1,-3), (qros(ii,6), ii=mp,1,-3)
c19    format(420e12.4)

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  subroutine tfilter
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine tfilter(ntime)
c
c   Smooths the temperature field, I think for outputting purposes.  This
c   subroutine is not called.
c
      parameter(nx=26,nxn=25,ny=30,nyn=29)

      common /coup2/ to(26,30),tave(26,30)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)
      common/filt/ tfil(26,30)
      dimension work(30)
c
c  first apply two passes of a 1-2-1 filter to the thermal
c  output to in the y direction. Since we don't want to
c  filter the results, but just smooth the output we put
c  the smoothed array in 'tfil'
c
c first y filter
      do 10 i=1,26
      work(1)=(2*to(i,1)+to(i,2))/3.
      work(30)=(2*to(i,30)+to(i,29))/3.
      do 20 j=2,29
  20  work(j)=(to(i,j-1)+2*to(i,j)+to(i,j+1))/4.
c second y filter
      tfil(i,1)=(2*work(1)+work(2))/3.
      tfil(i,30)=(2*work(30)+work(29))/3.
      do 30 j=2,29
  30  tfil(i,j)=(work(j-1)+2*work(j)+work(j+1))/4.
  10  continue
c
c now apply a 1-2-1 filter to to in the x direction
c
      do 40 j=1,30
      work(1)=(2*tfil(1,j)+tfil(2,j))/3.
      work(26)=(2*tfil(26,j)+tfil(25,j))/3.
      do 50 i=2,25
  50  work(i)=(tfil(i-1,j)+2*tfil(i,j)+tfil(i+1,j))/4.
      do 60 i=1,26
  60  tfil(i,j)=work(i)
  40  continue
c      write(xx)ntime,tfil
      return
      end

 
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  DATA BLOCKS
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      block data one 
      parameter(nx=26,nxn=25,ny=30,nyn=29)
      common /cl2/mthend(12),day(12)
      common /cl3/mthend3(48),day3(48)
      common/tem1/tn(nx,ny),w(nx,ny),tza(26),hbar(26)
c      data day/31.,30.,29.,31.,30.,31.,30.,31.,31.,30.,31.,30./
      data day/30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30./
c      data mthend/15,45,74,105,135,166,196,227,258,288,319,349/
      data mthend/15,45,75,105,135,165,195,225,255,285,315,345/
      data hbar/10*165.,172.,2*175.,165.,137.,110.,97.,89.,81.,
     1  73.,66.,60.,52.,3*50./ 
      data day3/30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,
     1 30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,
     1 30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,
     1 30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30.,30./
      data mthend3/15,45,75,105,135,165,195,225,255,285,315,345,
     1  375,405,435,465,495,525,555,585,615,645,675,705,
     1  735,765,795,825,855,885,915,945,975,1005,1035,1065,
     1  1095,1125,1155,1185,1215,1245,1275,1305,1335,1365,1395,1425/

      end
